@@ -50,7 +50,7 @@
 #include "slp_message.h"
 #include "slp_xmalloc.h"
 
-#ifndef WIN32
+#ifndef _WIN32
 #include <sys/types.h>
 #include <netinet/in.h>
 #endif
@@ -841,6 +841,65 @@ int ParseSrvTypeRply(SLPBuffer buffer, SLPSrvTypeRply* srvtyperply)
     return 0;
 }
 
+/*--------------------------------------------------------------------------*/
+int ParseExtension(SLPBuffer buffer, SLPMessage message)
+/* Parse extensions *after* all standard protocol fields are parsed         */
+/*--------------------------------------------------------------------------*/
+{
+    int             extid;
+    int             nextoffset;
+    int             result  = SLP_ERROR_OK;
+
+    nextoffset = message->header.extoffset;
+    while(nextoffset)
+    {
+        buffer->curpos = buffer->start + nextoffset;
+        if(buffer->curpos + 5 >= buffer->end)
+        {
+            /* Extension takes us past the end of the buffer */
+            result = SLP_ERROR_PARSE_ERROR;
+            goto CLEANUP;
+        }
+    
+        extid = AsUINT16(buffer->curpos);
+        buffer->curpos += 2;
+
+        nextoffset = AsUINT24(buffer->curpos);
+        buffer->curpos += 3;
+        
+        switch(extid)
+        {
+        case SLP_EXTENSION_ID_REG_PID:
+            if(message->header.functionid == SLP_FUNCT_SRVREG)
+            {
+                /* check to see if buffer is large enough to contain the 4 byte pid */
+                if(buffer->curpos + 4 > buffer->end)
+                {
+                    result = SLP_ERROR_PARSE_ERROR;
+                    goto CLEANUP;
+                }
+                
+                message->body.srvreg.pid = AsUINT32(buffer->curpos);
+                buffer->curpos += 4;
+            }
+            break;
+
+        default:
+            if (extid >= 0x4000 && extid <= 0x7FFF )
+            {
+                /* This is a required extension.  We better error out */
+                result = SLP_ERROR_MESSAGE_NOT_SUPPORTED;
+                goto CLEANUP;
+            }
+            break;
+        }
+    }
+
+CLEANUP:
+    
+    return result;
+}
+
 /*=========================================================================*/
 void SLPMessageFreeInternals(SLPMessage message)
 /*=========================================================================*/
@@ -1064,12 +1123,16 @@ int SLPMessageParseBuffer(struct sockaddr_in* peerinfo,
         }
     }
 
+    if(result == 0 && message->header.extoffset)
+    {
+        result = ParseExtension(buffer,message);
+    }
+
     return result;
 }
 
-/*=========================================================================*/
+#if !defined(i386)
 /* Functions used to parse buffers                                         */
-
 /*-------------------------------------------------------------------------*/
 unsigned short AsUINT16(const char *charptr)
 /*-------------------------------------------------------------------------*/
@@ -1093,11 +1156,10 @@ unsigned int AsUINT32(const char *charptr)
     unsigned char *ucp = (unsigned char *) charptr;
     return(ucp[0] << 24) | (ucp[1] << 16) | (ucp[2] << 8) | ucp[3]; 
 }
-
+/*=========================================================================*/
 
 /*=========================================================================*/
 /* Functions used to set buffers                                           */
-
 /*-------------------------------------------------------------------------*/
 void ToUINT16(char *charptr, unsigned int val)
 /*-------------------------------------------------------------------------*/
@@ -1124,4 +1186,4 @@ void ToUINT32(char *charptr, unsigned int val)
     charptr[2] = (val >> 8) & 0xff;
     charptr[3] = val & 0xff;
 }
-
+#endif
