@@ -52,21 +52,32 @@
 
 #include "slp_buffer.h"
 
-#ifdef WIN32
+#ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
+#include <winsock.h>
+#ifndef UINT32_T_DEFINED
+#define UINT32_T_DEFINED
+typedef unsigned int uint32_t;
+#endif
 #else
 #include <sys/types.h>
 #ifdef __FreeBSD__
 #include <sys/param.h>			/* for limits.h - INT_MAX slpd*/
 #endif
 #include <netinet/in.h>			/* for htonl() routines */
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#else
+#include <inttypes.h>
+#endif
 #endif
 
 typedef char            CHAR;
 typedef unsigned char   UINT8;
 typedef unsigned short  UINT16;
 
-#ifndef WIN32
+#ifndef _WIN32
 typedef unsigned long   UINT32;
 #endif
 
@@ -148,6 +159,7 @@ typedef UINT32*         PUINT32;
 #define SLP_DA_SERVICE_TYPE       "service:directory-agent"
 #define SLP_SA_SERVICE_TYPE       "service:service-agent"
 
+
 /*=========================================================================*/
 /* SLP Registration Sources                                                */
 /*=========================================================================*/
@@ -157,6 +169,12 @@ typedef UINT32*         PUINT32;
 #define SLP_REG_SOURCE_STATIC   3  /* from the slp.reg file */
 
 
+/*=========================================================================*/
+/* SLP Extension IDs                                                       */
+/*=========================================================================*/
+#define SLP_EXTENSION_ID_REG_PID   0x9799
+
+
 /*=========================================================================*/
 /* SLPHeader structure and associated functions                            */
 /*=========================================================================*/
@@ -252,6 +270,8 @@ typedef struct _SLPSrvReg
     const char*         attrlist;
     int                 authcount;
     SLPAuthBlock*       autharray;
+    /* The following are used for OpenSLP specific extensions */
+    uint32_t            pid;
     /* The following are not part of the RFC protocol.  They are used by   */
     /* the OpenSLP implementation for convenience                          */
     int                 source;
@@ -366,6 +386,7 @@ typedef struct _SLPSAAdvert
 }SLPSAAdvert;
 
 
+
 /*=========================================================================*/
 typedef struct _SLPMessage
 /*=========================================================================*/
@@ -451,6 +472,33 @@ int SLPMessageParseBuffer(struct sockaddr_in* peerinfo,
 /*            pointers in SLPMessage will be invalidated.                  */
 /*=========================================================================*/
 
+#ifdef i386
+
+
+/*=========================================================================*/
+/* Macros used to parse buffers                                            */
+#define AsUINT16(charptr)   ( ntohs(*((PUINT16)(charptr))) )
+#define AsUINT24(charptr)   ( ntohl(*((PUINT32)(charptr)))>>8 )
+#define AsUINT32(charptr)   ( ntohl(*((PUINT32)(charptr))) )
+/*=========================================================================*/
+
+
+/*=========================================================================*/
+/* Macros used to set buffers                                              */
+#define ToUINT16(charptr,val)   ( *((PUINT16)(charptr)) =  htons(((UINT16)val)) )
+#define ToUINT24(charptr,val)   ( *((PUINT32)(charptr)) =  htonl((val)<<8) )
+#define ToUINT32(charptr,val)   ( *((PUINT32)(charptr)) =  htonl((val)) )
+/*=========================================================================*/
+
+
+/*=========================================================================*/
+/* Macros to check in_addr                                                 */
+#define ISLOCAL(addr) ((ntohl((addr).s_addr) & 0xff000000) == 0x7f000000)
+#define ISMCAST(addr) ((ntohl((addr).s_addr) & 0xff000000) >= 0xef000000)
+/*=========================================================================*/
+
+#else 
+
 /*=========================================================================*/
 /* Functions used to parse buffers                                         */
 unsigned short AsUINT16(const char *charptr);
@@ -465,17 +513,6 @@ void ToUINT24(char *charptr, unsigned int val);
 void ToUINT32(char *charptr, unsigned int val);
 /*=========================================================================*/
 
-
-#ifdef i386
-
-/*=========================================================================*/
-/* Macros to check in_addr                                                 */
-#define ISLOCAL(addr) ((ntohl((addr).s_addr) & 0xff000000) == 0x7f000000)
-#define ISMCAST(addr) ((ntohl((addr).s_addr) & 0xff000000) >= 0xef000000)
-/*=========================================================================*/
-
-#else 
-
 /*=========================================================================*/
 /* Macros to check in_addr                                                 */
 #define ISLOCAL(addr) (((addr).s_addr & 0xff000000) == 0x7f000000)
