@@ -267,9 +267,9 @@ SLPSpiHandle SLPSpiOpen(const char* spifile, int cacheprivate)
     fp = fopen(spifile,"r");
     if(fp)
     {
-        result = xmalloc(sizeof(struct _SLPSpiHandle));
+        result = xmalloc(sizeof(structSLPSpiHandle));
         if(result == 0) return 0;
-        memset(result, 0, sizeof(struct _SLPSpiHandle));
+        memset(result, 0, sizeof(structSLPSpiHandle));
         
         result->spifile = xstrdup(spifile);
         result->cacheprivate = cacheprivate;
