@@ -50,7 +50,8 @@
 #if(!defined LIBSLP_H_INCLUDED)
 #define LIBSLP_H_INCLUDED
 
-#ifdef WIN32
+#ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -82,6 +83,10 @@
 #include "slp_database.h"
 #include "slp_compare.h"
 #include "slp_xmalloc.h"
+#include "slp_parse.h"
+#include "slp_iface.h"
+#include "slp_xcast.h"
+#include "slp_pid.h"
 #ifdef ENABLE_SLPv2_SECURITY
 #include "slp_auth.h"
 #include "slp_spi.h"
@@ -92,7 +97,7 @@
 #define SLP_FUNCT_DASRVRQST         0x7f   /* fake id used internally */
 
 #if(!defined LIBSLP_CONFFILE)
-#ifdef WIN32
+#ifdef _WIN32
 #define LIBSLP_CONFFILE "%WINDIR%\\slp.conf"
 #define LIBSLP_SPIFILE  "%WINDIR%\\slp.spi"
 #else
@@ -231,7 +236,7 @@ typedef union _SLPHandleCallParams
 /*=========================================================================*/
 typedef struct _SLPHandleInfo
 /* The SLPHandle that is used internally in slplib is actually a pointer to*/
-/* a struct _SLPHandleInfo                                                 */
+/* a structSLPHandleInfo                                                 */
 /*=========================================================================*/
 {
     unsigned int        sig;
@@ -379,6 +384,7 @@ typedef SLPBoolean NetworkRplyCallback(SLPError errorcode,
 SLPError NetworkRqstRply(int sock,
                          struct sockaddr_in* peeraddr,
                          const char* langtag,
+                         int extoffset,
                          char* buf,
                          char buftype,
                          int bufsize,
@@ -390,6 +396,36 @@ SLPError NetworkRqstRply(int sock,
 /*=========================================================================*/
 
 
+/*=========================================================================*/ 
+SLPError NetworkMcastRqstRply(const char* langtag,
+                              char* buf,
+                              char buftype,
+                              int bufsize,
+                              NetworkRplyCallback callback,
+                              void * cookie);
+/* Description:                                                            */
+/*                                                                         */
+/* Broadcasts/multicasts SLP messages via multicast convergence algorithm  */
+/*                                                                         */
+/* langtag  (IN) Language tag to use in SLP message header                 */
+/*                                                                         */
+/* buf      (IN) pointer to the portion of the SLP message to send. The    */
+/*               portion to that should be pointed to is everything after  */
+/*               the pr-list. NetworkXcastRqstRply() automatically         */
+/*               generates the header and the prlist.                      */
+/*                                                                         */
+/* buftype  (IN) the function-id to use in the SLPMessage header           */
+/*                                                                         */
+/* bufsize  (IN) the size of the buffer pointed to by buf                  */
+/*                                                                         */
+/* callback (IN) the callback to use for reporting results                 */
+/*                                                                         */
+/* cookie   (IN) the cookie to pass to the callback                        */
+/*                                                                         */
+/* Returns  -    SLP_OK on success. SLP_ERROR on failure                   */
+/*=========================================================================*/ 
+
+
 /*=========================================================================*/
 int KnownDAConnect(PSLPHandleInfo handle,
                    int scopelistlen,
