@@ -192,34 +192,37 @@ SLPError ProcessSrvDeReg(PSLPHandleInfo handle)
     /* TODO: No tag list for now put in taglist stuff */
     ToUINT16(curpos,0);
 
-
+   
     /*--------------------------*/
     /* Call the RqstRply engine */
     /*--------------------------*/
     sock = NetworkConnectToSA(handle,
-                              handle->params.dereg.scopelist,
-                              handle->params.dereg.scopelistlen,
+                              handle->params.reg.scopelist,
+                              handle->params.reg.scopelistlen,
                               &peeraddr);
     if(sock >= 0)
     {
         result = NetworkRqstRply(sock,
-                                 &peeraddr,
-                                 handle->langtag,
-                                 buf,
-                                 SLP_FUNCT_SRVDEREG,
-                                 bufsize,
-                                 CallbackSrvDeReg,
-                                 handle);
+				 &peeraddr,
+				 handle->langtag,
+				 0,
+				 buf,
+				 SLP_FUNCT_SRVDEREG,
+				 bufsize,
+				 CallbackSrvDeReg,
+				 handle);
         if (result)
         {
             NetworkDisconnectSA(handle);
-        }   
+        }           
     }
     else
     {
         result = SLP_NETWORK_INIT_FAILED;
+        goto FINISHED;
     }
-
+    
+    
     FINISHED:
     if(buf) xfree(buf);
 #ifdef ENABLE_SLPv2_SECURITY
@@ -243,7 +246,7 @@ SLPError AsyncProcessSrvDeReg(PSLPHandleInfo handle)
 #endif
 
 /*=========================================================================*/
-SLPError SLPDereg(SLPHandle  hSLP,
+SLPError SLPAPI SLPDereg(SLPHandle  hSLP,
                   const char *srvUrl,
                   SLPRegReport callback,
                   void *cookie)
