@@ -223,27 +223,19 @@ SLPError ProcessAttrRqst(PSLPHandleInfo handle)
         if(sock == -1)
         {
             /* use multicast as a last resort */
-            sock = NetworkConnectToMulticast(&peeraddr);
-            result = NetworkRqstRply(sock,
-                                     &peeraddr,
-                                     handle->langtag,
-                                     buf,
-                                     SLP_FUNCT_ATTRRQST,
-                                     bufsize,
-                                     ProcessAttrRplyCallback,
-                                     handle);
-            #ifdef WIN32
-            closesocket(sock);
-            #else
-            close(sock);
-            #endif
-
+            result = NetworkMcastRqstRply(handle->langtag,
+                                          buf,
+                                          SLP_FUNCT_ATTRRQST,
+                                          bufsize,
+                                          ProcessAttrRplyCallback,
+                                          handle);
             break;
         }
 
         result = NetworkRqstRply(sock,
                                  &peeraddr,
                                  handle->langtag,
+                                 0,
                                  buf,
                                  SLP_FUNCT_ATTRRQST,
                                  bufsize,
@@ -282,7 +274,7 @@ SLPError AsyncProcessAttrRqst(PSLPHandleInfo handle)
 
 
 /*=========================================================================*/
-SLPError SLPFindAttrs(SLPHandle   hSLP,
+SLPError SLPAPI SLPFindAttrs(SLPHandle   hSLP,
                       const char *pcURLOrServiceType,
                       const char *pcScopeList,
                       const char *pcAttrIds,
