@@ -382,27 +382,19 @@ SLPError ProcessSrvRqst(PSLPHandleInfo handle)
         if(sock == -1)
         {
             /* use multicast as a last resort */
-            sock = NetworkConnectToMulticast(&peeraddr);
-            result = NetworkRqstRply(sock,
-                                     &peeraddr,
-                                     handle->langtag,
-                                     buf,
-                                     SLP_FUNCT_SRVRQST,
-                                     bufsize,
-                                     ProcessSrvRplyCallback,
-                                     handle);
-            /* close the socket */
-            #ifdef WIN32
-            closesocket(sock);
-            #else
-            close(sock);
-            #endif
+            result = NetworkMcastRqstRply(handle->langtag,
+                                          buf,
+                                          SLP_FUNCT_SRVRQST,
+                                          bufsize,
+                                          ProcessSrvRplyCallback,
+                                          handle);
             break;
         }
 
         result = NetworkRqstRply(sock,
                                  &peeraddr,
                                  handle->langtag,
+                                 0,
                                  buf,
                                  SLP_FUNCT_SRVRQST,
                                  bufsize,
@@ -441,7 +433,7 @@ SLPError AsyncProcessSrvRqst(PSLPHandleInfo handle)
 
 
 /*=========================================================================*/
-SLPError SLPFindSrvs(SLPHandle  hSLP,
+SLPError SLPAPI SLPFindSrvs(SLPHandle  hSLP,
                      const char *pcServiceType,
                      const char *pcScopeList,
                      const char *pcSearchFilter,
