@@ -57,7 +57,7 @@ SLPBoolean ColateSrvTypeCallback(SLPHandle hSLP,
 {
     PSLPHandleInfo          handle;
     SLPBoolean              result;
-    size_t                  srvtypeslen;
+    int		                srvtypeslen;
     char*                   srvtypes;
     
     handle = (PSLPHandleInfo) hSLP;
@@ -127,7 +127,7 @@ SLPBoolean ColateSrvTypeCallback(SLPHandle hSLP,
                                   strlen(pcSrvTypes),
                                   pcSrvTypes,
                                   &srvtypeslen,
-                                  srvtypes) != (int)srvtypeslen)
+                                  srvtypes) != srvtypeslen)
             {
                 xfree(handle->collatedsrvtypes);
                 handle->collatedsrvtypes = srvtypes;
@@ -271,26 +271,19 @@ SLPError ProcessSrvTypeRqst(PSLPHandleInfo handle)
         if(sock == -1)
         {
             /* use multicast as a last resort */
-            sock = NetworkConnectToMulticast(&peeraddr);
-            result = NetworkRqstRply(sock,
-                                     &peeraddr,
-                                     handle->langtag,
-                                     buf,
-                                     SLP_FUNCT_SRVTYPERQST,
-                                     bufsize,
-                                     ProcessSrvTypeRplyCallback,
-                                     handle);
-            #ifdef WIN32
-            closesocket(sock);
-            #else
-            close(sock);
-            #endif
+            result = NetworkMcastRqstRply(handle->langtag,
+                                          buf,
+                                          SLP_FUNCT_SRVTYPERQST,
+                                          bufsize,
+                                          ProcessSrvTypeRplyCallback,
+                                          handle);
             break;
         }
 
         result = NetworkRqstRply(sock,
                                  &peeraddr,
                                  handle->langtag,
+                                 0,
                                  buf,
                                  SLP_FUNCT_SRVTYPERQST,
                                  bufsize,
@@ -324,7 +317,7 @@ SLPError AsyncProcessSrvTypeRqst(PSLPHandleInfo handle)
 #endif
 
 /*=========================================================================*/
-SLPError SLPFindSrvTypes(SLPHandle    hSLP,
+SLPError SLPAPI SLPFindSrvTypes(SLPHandle    hSLP,
                          const char  *pcNamingAuthority,
                          const char  *pcScopeList,
                          SLPSrvTypeCallback callback,
