@@ -51,41 +51,6 @@
 #include "libslp.h"
 
 
-/*=========================================================================*/
-int NetworkConnectToMulticast(struct sockaddr_in* peeraddr)
-/*=========================================================================*/
-{
-    int                 sock = -1;
-    struct in_addr      iface;
-    struct in_addr      *ifaceptr = NULL;
-
-    if(SLPPropertyAsBoolean(SLPGetProperty("net.slp.isBroadcastOnly")) == 0)
-    {
-        if (SLPGetProperty("net.slp.multicastIF"))
-        {
-            if( inet_aton(SLPGetProperty("net.slp.multicastIF"), &iface) )
-            {
-                ifaceptr = &iface;
-            } 
-            else 
-            {
-                return -1;
-            }
-        }
-
-        sock = SLPNetworkConnectToMulticast(peeraddr,
-                                            atoi(SLPGetProperty("net.slp.multicastTTL")),
-                                            ifaceptr);
-    }
-
-    if(sock < 0)
-    {
-        sock = SLPNetworkConnectToBroadcast(peeraddr);
-    }
-
-    return sock;    
-}
-
 /*=========================================================================*/
 int NetworkConnectToSlpd(struct sockaddr_in* peeraddr)
 /* Connects to slpd and provides a peeraddr to send to                     */
@@ -95,7 +60,7 @@ int NetworkConnectToSlpd(struct sockaddr_in* peeraddr)
 /* Returns          Connected socket or -1 if no DA connection can be made */
 /*=========================================================================*/
 {
-#ifdef WIN32 /* on WIN32 setsockopt takes a const char * argument */
+#ifdef _WIN32 /* on WIN32 setsockopt takes a const char * argument */
     char lowat;
 #else
     int lowat;
@@ -124,11 +89,7 @@ int NetworkConnectToSlpd(struct sockaddr_in* peeraddr)
         else
         {
             /* Could not connect to the slpd through the loopback */
-            #ifdef WIN32
-            closesocket(result);
-            #else
             close(result);
-            #endif
             result = -1;
         }
     }
@@ -145,12 +106,7 @@ void NetworkDisconnectDA(PSLPHandleInfo handle)
 {
     if(handle->dasock)
     {
-        #ifdef WIN32
-        closesocket(handle->dasock);
-        #else
         close(handle->dasock);
-        #endif
-        
         handle->dasock = -1;
     }
 
@@ -168,11 +124,7 @@ void NetworkDisconnectSA(PSLPHandleInfo handle)
 {
     if(handle->sasock)
     {
-        #ifdef WIN32
-        closesocket(handle->sasock);
-        #else
         close(handle->sasock);
-        #endif
         handle->sasock = -1;
     }
 }
@@ -215,12 +167,7 @@ int NetworkConnectToDA(PSLPHandleInfo handle,
         /* close handle cause it can't support the scope */
         if(handle->dasock >= 0)
         {
-            #ifdef WIN32
-            closesocket(handle->dasock);
-            #else
             close(handle->dasock);
-            #endif
-            
         }
 
         /* Attempt to connect to DA that does support the scope */
@@ -278,11 +225,7 @@ int NetworkConnectToSA(PSLPHandleInfo handle,
         /* close handle cause it can't support the scope */
         if(handle->sasock >= 0)
         {
-            #ifdef WIN32
-            closesocket(handle->sasock);
-            #else
             close(handle->sasock);
-            #endif
         }
 
         /*-----------------------------------------*/
@@ -306,10 +249,12 @@ int NetworkConnectToSA(PSLPHandleInfo handle,
 }
 
 
+
 /*=========================================================================*/ 
 SLPError NetworkRqstRply(int sock,
                          struct sockaddr_in* destaddr,
                          const char* langtag,
+                         int extoffset,
                          char* buf,
                          char buftype,
                          int bufsize,
@@ -336,7 +281,7 @@ SLPError NetworkRqstRply(int sock,
     int                 rplycount       = 0;
     int                 maxwait         = 0;
     int                 totaltimeout    = 0;
-#ifdef WIN32 /* on WIN32 setsockopt takes a const char * argument */
+#ifdef _WIN32 /* on WIN32 setsockopt takes a const char * argument */
     char                socktype        = 0;
 #else
     int                 socktype        = 0;
@@ -495,7 +440,18 @@ SLPError NetworkRqstRply(int sock,
         }
         ToUINT16(sendbuf->start + 5, flags);
         /*ext offset*/
-        ToUINT24(sendbuf->start + 7,0);
+        /* TRICKY: the extoffset passed into us was the offset not
+         * including the header.  We need to fix up the offset so
+         * that it is from the beginning of the SLP message
+         */
+        if(extoffset != 0)
+	{
+            ToUINT24(sendbuf->start + 7,extoffset + langtaglen + 14);
+	}
+        else
+        {
+	    ToUINT24(sendbuf->start + 7, 0);
+	}
         /*xid*/
         ToUINT16(sendbuf->start + 10,xid);
         /*lang tag len*/
@@ -588,12 +544,28 @@ SLPError NetworkRqstRply(int sock,
                     /* SLP_FUNCT_SRVTYPERQST)                               */
                     if(prlist && socktype == SOCK_DGRAM)
                     {
-                        if(prlistlen != 0)
+                        /* calculate the peeraddr string and length */
+                        char* peeraddrstr = NULL;
+                        int peeraddrstrlen = 0;
+                        peeraddrstr = inet_ntoa(peeraddr.sin_addr);
+                        if(peeraddrstr)
                         {
-                            strcat(prlist,",");
+                            peeraddrstrlen = strlen(peeraddrstr);
+                            
+                            /* Append to the prlist if we won't overflow */
+                            if((prlistlen + peeraddrstrlen + 1) < mtu )
+                            {
+                                /* append comma if necessary */
+                                if(prlistlen != 0)
+                                {
+                                    strcat(prlist,",");
+                                    prlistlen ++;
+                                }
+                                /* append address string */
+                                strcat(prlist,peeraddrstr);
+                                prlistlen += peeraddrstrlen;
+                            }
                         }
-                        strcat(prlist,inet_ntoa(peeraddr.sin_addr));
-                        prlistlen =  strlen(prlist);
                     }
                 }
             }
@@ -634,3 +606,299 @@ SLPError NetworkRqstRply(int sock,
     
     return result;
 }
+
+/*=========================================================================*/ 
+SLPError NetworkMcastRqstRply(const char* langtag,
+                              char* buf,
+                              char buftype,
+                              int bufsize,
+                              NetworkRplyCallback callback,
+                              void * cookie)
+/* Description:                                                            */
+/*                                                                         */
+/* Broadcasts/multicasts SLP messages via multicast convergence algorithm  */
+/*                                                                         */
+/* langtag  (IN) Language tag to use in SLP message header                 */
+/*                                                                         */
+/* buf      (IN) pointer to the portion of the SLP message to send. The    */
+/*               portion to that should be pointed to is everything after  */
+/*               the pr-list. NetworkXcastRqstRply() automatically         */
+/*               generates the header and the prlist.                      */
+/*                                                                         */
+/* buftype  (IN) the function-id to use in the SLPMessage header           */
+/*                                                                         */
+/* bufsize  (IN) the size of the buffer pointed to by buf                  */
+/*                                                                         */
+/* callback (IN) the callback to use for reporting results                 */
+/*                                                                         */
+/* cookie   (IN) the cookie to pass to the callback                        */
+/*                                                                         */
+/* Returns  -    SLP_OK on success. SLP_ERROR on failure                   */
+/*=========================================================================*/ 
+{
+    struct timeval      timeout;
+    struct sockaddr_in  peeraddr;
+    SLPBuffer           sendbuf         = 0;
+    SLPBuffer           recvbuf         = 0;
+    SLPError            result          = 0;
+    int                 langtaglen      = 0;
+    int                 prlistlen       = 0;
+    char*               prlist          = 0;
+    int                 xid             = 0;
+    int                 mtu             = 0;
+    int                 size            = 0;
+    int                 xmitcount       = 0;
+    int                 rplycount       = 0;
+    int                 maxwait         = 0;
+    int                 totaltimeout    = 0;
+    int                 usebroadcast    = 0;
+    int                 timeouts[MAX_RETRANSMITS];
+    SLPIfaceInfo        ifaceinfo;
+    SLPXcastSockets     xcastsocks;
+
+#ifdef DEBUG
+    /* This function only supports multicast or broadcast of the following
+     *  messages
+     */
+    if(buftype != SLP_FUNCT_SRVRQST &&
+       buftype != SLP_FUNCT_ATTRRQST &&
+       buftype != SLP_FUNCT_SRVTYPERQST &&
+       buftype != SLP_FUNCT_DASRVRQST)
+    {
+        return SLP_PARAMETER_BAD;
+    }
+#endif
+
+    /*----------------------------------------------------*/
+    /* Save off a few things we don't want to recalculate */
+    /*----------------------------------------------------*/
+    langtaglen = strlen(langtag);
+    xid = SLPXidGenerate();
+    mtu = SLPPropertyAsInteger(SLPGetProperty("net.slp.MTU"));
+    sendbuf = SLPBufferAlloc(mtu);
+    if(sendbuf == 0)
+    {
+        result = SLP_MEMORY_ALLOC_FAILED;
+        goto FINISHED;
+    }
+    if(SLPIfaceGetInfo(SLPGetProperty("net.slp.interfaces"),&ifaceinfo))
+    {
+        result = SLP_NETWORK_ERROR;
+        goto FINISHED;
+    }
+    usebroadcast = SLPPropertyAsBoolean(SLPGetProperty("net.slp.useBroadcast"));
+
+    /*-----------------------------------*/
+    /* Multicast/broadcast wait timeouts */
+    /*-----------------------------------*/
+    maxwait = SLPPropertyAsInteger(SLPGetProperty("net.slp.multicastMaximumWait"));
+    SLPPropertyAsIntegerVector(SLPGetProperty("net.slp.multicastTimeouts"), 
+                               timeouts, 
+                               MAX_RETRANSMITS );
+
+    /* Special case for fake SLP_FUNCT_DASRVRQST */
+    if(buftype == SLP_FUNCT_DASRVRQST)
+    {
+        /* do something special for SRVRQST that will be discovering DAs */
+        maxwait = SLPPropertyAsInteger(SLPGetProperty("net.slp.DADiscoveryMaximumWait"));
+        SLPPropertyAsIntegerVector(SLPGetProperty("net.slp.DADiscoveryTimeouts"),
+                                   timeouts,
+                                   MAX_RETRANSMITS );
+        /* SLP_FUNCT_DASRVRQST is a fake function.  We really want to */
+        /* send a SRVRQST                                             */
+        buftype  = SLP_FUNCT_SRVRQST;
+    }
+
+    /*---------------------------------------------------------------------*/
+    /* Allocate memory for the prlist for appropriate messages.            */
+    /* Notice that the prlist is as large as the MTU -- thus assuring that */
+    /* there will not be any buffer overwrites regardless of how many      */
+    /* previous responders there are.   This is because the retransmit     */
+    /* code terminates if ever MTU is exceeded for any datagram message.   */
+    /*---------------------------------------------------------------------*/
+    prlist = (char*)xmalloc(mtu);
+    if(prlist == 0)
+    {
+        result = SLP_MEMORY_ALLOC_FAILED;
+        goto FINISHED;
+    }
+    *prlist = 0;
+    prlistlen = 0; 
+
+    /*--------------------------*/
+    /* Main retransmission loop */
+    /*--------------------------*/
+    xmitcount = 0;
+    while(xmitcount <= MAX_RETRANSMITS)
+    {
+        xmitcount++;
+
+        totaltimeout += timeouts[xmitcount];
+        if(totaltimeout >= maxwait ||  timeouts[xmitcount] == 0)
+        {
+            /* we are all done */
+            break;
+        }
+        timeout.tv_sec = timeouts[xmitcount] / 1000;
+        timeout.tv_usec = (timeouts[xmitcount] % 1000) * 1000;
+        
+        /*------------------------------------------------------------------*/
+        /* re-allocate buffer and make sure that the send buffer does not   */
+        /* exceed MTU for datagram transmission                             */
+        /*------------------------------------------------------------------*/
+        size = 14 + langtaglen + bufsize;
+        if(buftype == SLP_FUNCT_SRVRQST ||
+           buftype == SLP_FUNCT_ATTRRQST ||
+           buftype == SLP_FUNCT_SRVTYPERQST)
+        {
+            /* add in room for the prlist */
+            size += 2 + prlistlen;
+        }
+        if(size > mtu)
+        {
+            if(xmitcount == 0)
+            {
+                result = SLP_BUFFER_OVERFLOW;
+            }
+            goto FINISHED;
+        }
+        if((sendbuf = SLPBufferRealloc(sendbuf,size)) == 0)
+        {
+            result = SLP_MEMORY_ALLOC_FAILED;
+            goto FINISHED;
+        }
+
+        /*-----------------------------------*/
+        /* Add the header to the send buffer */
+        /*-----------------------------------*/
+        /*version*/
+        *(sendbuf->start)       = 2;
+        /*function id*/
+        *(sendbuf->start + 1)   = buftype;
+        /*length*/
+        ToUINT24(sendbuf->start + 2, size);
+        /*flags*/
+        ToUINT16(sendbuf->start + 5, SLP_FLAG_MCAST);
+        /*ext offset*/
+        ToUINT24(sendbuf->start + 7,0);
+        /*xid*/
+        ToUINT16(sendbuf->start + 10,xid);
+        /*lang tag len*/
+        ToUINT16(sendbuf->start + 12,langtaglen);
+        /*lang tag*/
+        memcpy(sendbuf->start + 14, langtag, langtaglen);
+        sendbuf->curpos = sendbuf->start + langtaglen + 14 ;
+
+        /*-----------------------------------*/
+        /* Add the prlist to the send buffer */
+        /*-----------------------------------*/
+        if(prlist)
+        {
+            ToUINT16(sendbuf->curpos,prlistlen);
+            sendbuf->curpos = sendbuf->curpos + 2;
+            memcpy(sendbuf->curpos, prlist, prlistlen);
+            sendbuf->curpos = sendbuf->curpos + prlistlen;
+        }
+
+        /*-----------------------------*/
+        /* Add the rest of the message */
+        /*-----------------------------*/
+        memcpy(sendbuf->curpos, buf, bufsize);
+
+        /*----------------------*/
+        /* send the send buffer */
+        /*----------------------*/
+        if(usebroadcast)
+        {
+            result = SLPBroadcastSend(&ifaceinfo,sendbuf,&xcastsocks);
+        }
+        else
+        {
+            result = SLPMulticastSend(&ifaceinfo,sendbuf,&xcastsocks);
+        }
+        if(result != 0)
+        {
+            /* we could not send the message for some reason */
+            result = SLP_NETWORK_ERROR;    
+            goto FINISHED;
+        }
+
+        /*----------------*/
+        /* Main recv loop */
+        /*----------------*/
+        while(1)
+        {
+            if(SLPXcastRecvMessage(&xcastsocks,
+                                   &recvbuf,
+                                   &peeraddr,
+                                   &timeout) != 0)
+            {
+                /* An error occured while receiving the message        */
+                /* probably a just time out error. break for re-send.  */
+                if(errno == ETIMEDOUT)
+                {
+                    result = SLP_NETWORK_TIMED_OUT;
+                }
+                else
+                {
+                    result = SLP_NETWORK_ERROR;
+                }
+                break;
+            }
+            
+            /* Sneek in and check the XID */
+            if(AsUINT16(recvbuf->start+10) == xid)
+            {
+                rplycount += 1;
+
+                /* Call the callback with the result and recvbuf */
+                if(callback(result,&peeraddr,recvbuf,cookie) == SLP_FALSE)
+                {
+                    /* Caller does not want any more info */
+                    /* We are done!                       */
+                    goto CLEANUP;
+                }
+                 
+                /* add the peer to the previous responder list */
+                if(prlistlen != 0)
+                {
+                    strcat(prlist,",");
+                }
+                strcat(prlist,inet_ntoa(peeraddr.sin_addr));
+                prlistlen =  strlen(prlist);
+            }
+        }
+
+        SLPXcastSocketsClose(&xcastsocks);
+    }
+
+
+    FINISHED:
+    /*---------------------------------------------------------------------*/
+    /* Notify the callback with SLP_LAST_CALL so that they know we're done */
+    /*---------------------------------------------------------------------*/
+    if(rplycount || result == SLP_NETWORK_TIMED_OUT)
+    {
+        result = SLP_LAST_CALL;
+    }
+    
+    callback(result, NULL,NULL,cookie);
+
+    if(result == SLP_LAST_CALL)
+    {
+        result = SLP_OK;
+    }
+    
+    
+    CLEANUP:
+    /*----------------*/
+    /* Free resources */
+    /*----------------*/
+    if(prlist) xfree(prlist);
+    SLPBufferFree(sendbuf);
+    SLPBufferFree(recvbuf);
+    SLPXcastSocketsClose(&xcastsocks);
+    
+    return result;
+}
+
