@@ -50,12 +50,12 @@
 #include "slp.h"
 #include "libslp.h"
 
-#ifdef WIN32 /* on Win32 strncasecmp is named strnicmp, but behaves the same */
+#ifdef _WIN32 /* on Win32 strncasecmp is named strnicmp, but behaves the same */
     #define strncasecmp(String1, String2, Num) strnicmp(String1, String2, Num)
 #endif
 
 /*=========================================================================*/
-void SLPFree(void* pvMem)                                                  
+void SLPAPI SLPFree(void* pvMem)                                                  
 /*                                                                         */
 /* Frees memory returned from SLPParseSrvURL(), SLPFindScopes(),           */
 /* SLPEscape(), and SLPUnescape().                                         */
@@ -71,9 +71,8 @@ void SLPFree(void* pvMem)
     }
 }
 
-
 /*=========================================================================*/
-SLPError SLPParseSrvURL(const char *pcSrvURL,
+SLPError SLPAPI SLPParseSrvURL(const char *pcSrvURL,
                         SLPSrvURL** ppSrvURL)
 /*                                                                         */
 /* Parses the URL passed in as the argument into a service URL structure   */
@@ -100,87 +99,16 @@ SLPError SLPParseSrvURL(const char *pcSrvURL,
 /*          appropriate error code is returned.                            */
 /*=========================================================================*/
 {
-    char*   empty;   /* always points to an empty string */
-    char*   slider1; /* points to location in the SLPSrvURL buffer */
-    char*   slider2;
-    char*   slider3;
-
-    /* Check for bad parameters */
-    if(pcSrvURL == 0 ||
-       ppSrvURL == 0)
-    {
-        return SLP_PARAMETER_BAD;
-    }
-
-    /* Allocate memory and set up sliders */
-    *ppSrvURL = (SLPSrvURL*)xmalloc(strlen(pcSrvURL) + sizeof(SLPSrvURL) + 5);
-    /* +5 ensures space for 5 null terminations */
-    if(*ppSrvURL == 0)
+    int result = SLPParseSrvUrl(strlen(pcSrvURL),
+                                pcSrvURL,
+                                (SLPParsedSrvUrl**) ppSrvURL);
+    switch(result)
     {
+    case ENOMEM:
         return SLP_MEMORY_ALLOC_FAILED;
-    }
-    memset(*ppSrvURL,0,strlen(pcSrvURL) + sizeof(SLPSrvURL) + 5);    
-    slider1 = ((char*)*ppSrvURL) + sizeof(SLPSrvURL);
-    slider2 = slider3 = (char*)pcSrvURL;
-    
-    /* Set empty */
-    empty = slider1;
-    slider1 ++;
-     
-    /* parse out the service type */
-    slider3 = (char*)strstr(slider2,":/");
-    if(slider3 == 0)
-    {
-        xfree(*ppSrvURL);
-        *ppSrvURL = 0;
+    case EINVAL:
         return SLP_PARSE_ERROR;
-    }    
-    memcpy(slider1,slider2,slider3-slider2);
-    (*ppSrvURL)->s_pcSrvType = slider1;
-    slider1 += (slider3 - slider2) + 1;
-
-    /* parse out the host */
-    slider3 = slider2 = slider3 + 3; /* + 3 skips the "://" */
-    while(*slider3 && *slider3 != '/' && *slider3 != ':') slider3++;
-    if(slider3-slider2 < 1)
-    {
-        /* no host part (this is okay according to RFC2609) */
-        (*ppSrvURL)->s_pcHost = empty;
-    }
-    else
-    {
-        memcpy(slider1,slider2,slider3-slider2);
-        (*ppSrvURL)->s_pcHost = slider1;
-        slider1 += (slider3 - slider2) + 1;
-    }
-
-    /* parse out the port */
-    if(*slider3 == ':')
-    {
-        slider3 = slider2 = slider3 + 1; /* + 1 skips the ":" */
-        while(*slider3 && (*slider3 != '/' && *slider3 != ';')) slider3++;
-        memcpy(slider1,slider2,slider3-slider2);
-        (*ppSrvURL)->s_iPort = atoi(slider1);
-        slider1 += (slider3-slider2) + 1;
-    }
-
-    /* parse out the remainder of the url */
-    if(*slider3)
-    {
-        slider2 = slider3;
-        while(*slider3) slider3 ++;
-        memcpy(slider1,slider2,slider3-slider2);
-        (*ppSrvURL)->s_pcSrvPart = slider1;
-        slider1 += (slider3 - slider2) + 1;
-    }
-    else
-    {
-        /* no remainder portion */
-        (*ppSrvURL)->s_pcSrvPart = empty;
-    }
-
-    /* set  the net family to always be an empty string for IP */
-    (*ppSrvURL)->s_pcNetFamily = empty;
+    } 
 
     return SLP_OK;
 }
@@ -190,7 +118,7 @@ SLPError SLPParseSrvURL(const char *pcSrvURL,
 #define ESCAPE_CHARACTER			'\\'
 #define ESCAPE_CHARACTER_STRING		"\\"
 /*=========================================================================*/
-SLPError SLPEscape(const char* pcInbuf,
+SLPError SLPAPI SLPEscape(const char* pcInbuf,
                    char** ppcOutBuf,
                    SLPBoolean isTag)
 /*                                                                         */
@@ -304,7 +232,7 @@ SLPError SLPEscape(const char* pcInbuf,
 }
 
 /*=========================================================================*/
-SLPError SLPUnescape(const char* pcInbuf,
+SLPError SLPAPI SLPUnescape(const char* pcInbuf,
                      char** ppcOutBuf,
                      SLPBoolean isTag)
 /*                                                                         */
@@ -414,7 +342,7 @@ SLPError SLPUnescape(const char* pcInbuf,
 
 
 /*=========================================================================*/
-SLPError SLPParseAttrs(const char* pcAttrList,
+SLPError SLPAPI SLPParseAttrs(const char* pcAttrList,
                        const char *pcAttrId,
                        char** ppcAttrVal)
 /*                                                                         */
@@ -438,7 +366,7 @@ SLPError SLPParseAttrs(const char* pcAttrList,
 {
     const char* slider1;
     const char* slider2;
-    size_t      attridlen;
+    int			attridlen;
 
     /* Check for bad parameters */
     if(pcAttrList == 0 ||
@@ -465,7 +393,7 @@ SLPError SLPParseAttrs(const char* pcAttrList,
 
         while(*slider2 && *slider2 != '=' && *slider2 !=')') slider2++;
 
-        if((int)attridlen == slider2-slider1 && 
+        if(attridlen == slider2-slider1 && 
            strncasecmp(slider1, pcAttrId, slider2 - slider1) == 0)
         {
             /* found the attribute id */
