@@ -57,6 +57,20 @@ extern "C"
 {
 #endif
 
+#if defined(_WIN32) && defined(_MSC_VER)
+/* MSVC auto-exports, BCB uses .def file */
+# define SLPCALLBACK
+# ifdef LIBSLP_EXPORTS
+#  define SLPEXP __declspec(dllexport)
+# else
+#  define SLPEXP __declspec(dllimport)
+# endif
+# define SLPAPI
+#else
+# define SLPCALLBACK
+# define SLPEXP
+# define SLPAPI
+#endif
 
 /*==========================================================================*/
 /* lifetime values, in  seconds, that are frequently used.                  */
@@ -65,7 +79,6 @@ extern "C"
 #define SLP_LIFETIME_MAXIMUM 65535   /* 18 hours */
 
 
-
 /*==========================================================================*/
 /* SLPError                                                                 */
 /* ---------                                                                */
@@ -201,12 +214,11 @@ typedef int SLPError;
 /* in a registration do not match the service type template for         */
 /* the service.                                                         */
 
-
 /*==========================================================================*/
 /* SLPBoolean                                                               */
 /*------------                                                              */
 /* The SLPBoolean enum is used as a boolean flag.                           */
-typedef enum 
+typedef enum
 {
     SLP_FALSE = 0,
     SLP_TRUE = 1
@@ -267,9 +279,9 @@ typedef void* SLPHandle;
 
 
 /*=========================================================================*/
-typedef void SLPRegReport(SLPHandle hSLP,
+typedef void SLPCALLBACK SLPRegReport(SLPHandle hSLP,
                           SLPError errCode,
-                          void *pvCookie);                                 
+                          void *pvCookie);
 /*                                                                         */
 /* The SLPRegReport callback type is the type of the callback function     */
 /* to the SLPReg(), SLPDereg(), and SLPDelAttrs() functions.               */
@@ -285,7 +297,7 @@ typedef void SLPRegReport(SLPHandle hSLP,
 
 
 /*=========================================================================*/
-typedef SLPBoolean SLPSrvTypeCallback(SLPHandle hSLP,
+typedef SLPBoolean SLPCALLBACK SLPSrvTypeCallback(SLPHandle hSLP,
                                       const char* pcSrvTypes,
                                       SLPError errCode,
                                       void *pvCookie);
@@ -317,7 +329,7 @@ typedef SLPBoolean SLPSrvTypeCallback(SLPHandle hSLP,
 
 
 /*=========================================================================*/
-typedef SLPBoolean SLPSrvURLCallback(SLPHandle hSLP,
+typedef SLPBoolean SLPCALLBACK SLPSrvURLCallback(SLPHandle hSLP,
                                      const char* pcSrvURL,
                                      unsigned short sLifetime,
                                      SLPError errCode,
@@ -357,7 +369,7 @@ typedef SLPBoolean SLPSrvURLCallback(SLPHandle hSLP,
 
 
 /*=========================================================================*/
-typedef SLPBoolean SLPAttrCallback(SLPHandle hSLP,
+typedef SLPBoolean SLPCALLBACK SLPAttrCallback(SLPHandle hSLP,
                                    const char* pcAttrList,
                                    SLPError errCode,
                                    void *pvCookie); 
@@ -406,7 +418,9 @@ typedef SLPBoolean SLPAttrCallback(SLPHandle hSLP,
 
 
 /*=========================================================================*/
-SLPError SLPOpen(const char *pcLang, SLPBoolean isAsync, SLPHandle *phSLP);
+SLPEXP SLPError SLPAPI SLPOpen(const char *pcLang,
+                             SLPBoolean isAsync,
+                             SLPHandle *phSLP);
 /*                                                                         */
 /* Returns a SLPHandle handle in the phSLP parameter for the language      */
 /* locale passed in as the pcLang parameter.  The client indicates if      */
@@ -445,7 +459,7 @@ SLPError SLPOpen(const char *pcLang, SLPBoolean isAsync, SLPHandle *phSLP);
 
 
 /*=========================================================================*/
-void SLPClose(SLPHandle hSLP);                                             
+SLPEXP void SLPAPI SLPClose(SLPHandle hSLP);
 /*                                                                         */
 /* Frees all resources associated with the handle.  If the handle was      */
 /* invalid, the function returns silently.  Any outstanding synchronous    */
@@ -455,14 +469,16 @@ void SLPClose(SLPHandle hSLP);
 /* SLPHandle    A SLPHandle handle returned from a call to SLPOpen().      */
 /*=========================================================================*/
 
+#define SLP_REG_FLAG_FRESH      (1)
+#define SLP_REG_FLAG_WATCH_PID  (1 << 1)
 
 /*=========================================================================*/
-SLPError SLPReg(SLPHandle   hSLP,
+SLPEXP SLPError SLPAPI SLPReg(SLPHandle   hSLP,
                 const char  *pcSrvURL,
                 const unsigned short usLifetime,
                 const char  *pcSrvType,
                 const char  *pcAttrs,
-                SLPBoolean  fresh,
+                SLPBoolean fresh,
                 SLPRegReport callback,
                 void *pvCookie); 
 /*                                                                         */
@@ -492,7 +508,9 @@ SLPError SLPReg(SLPHandle   hSLP,
 /*              integer less than or equal to SLP_LIFETIME_MAXIMUM and     */
 /*              greater than zero. If SLP_LIFETIME_MAXIMUM is used, the    */
 /*              registration will remain for the life of the calling       */
-/*              process.                                                   */
+/*              process.  Also, OpenSLP, will not allow registrations to   */
+/*              be made with SLP_LIFETIME_MAXIMUM unless                   */
+/*              SLP_REG_FLAG_WATCH_PID is also used                        */
 /*                                                                         */
 /* pcSrvType    This parameter is ALWAYS ignored since the SLP Service URL */
 /*              syntax required for the pcSrvURL encapsulates the service  */
@@ -502,10 +520,9 @@ SLPError SLPReg(SLPHandle   hSLP,
 /*              for the attributes of the advertisement.  Use empty string,*/
 /*              "" for no attributes.                                      */
 /*                                                                         */
-/* fresh        An SLPBoolean that is SLP_TRUE if the registration is new  */
-/*              or SLP_FALSE if a reregistration.  OpenSLP does not support*/
-/*              incremental registrations.  If SLP_FALSE is passed in, an  */
-/*              SLP_NOT_IMPLEMENTED error will be returned.                */
+/* fresh        Use of non-fresh registrations is deprecated.  SLP_TRUE    */
+/*              must be passed in for this parameter or SLP_BAD_PARAMETER  */
+/*              will be returned                                           */
 /*                                                                         */
 /* callback     A SLPRegReport callback to report the operation completion */
 /*              status.                                                    */
@@ -519,7 +536,7 @@ SLPError SLPReg(SLPHandle   hSLP,
 
 
 /*=========================================================================*/
-SLPError SLPDereg(SLPHandle  hSLP,
+SLPEXP SLPError SLPAPI SLPDereg(SLPHandle  hSLP,
                   const char *pcSrvURL,
                   SLPRegReport callback,
                   void *pvCookie);   
@@ -546,9 +563,8 @@ SLPError SLPDereg(SLPHandle  hSLP,
 /*=========================================================================*/
 
 
-
 /*=========================================================================*/
-SLPError SLPDelAttrs(SLPHandle   hSLP,
+SLPEXP SLPError SLPAPI SLPDelAttrs(SLPHandle   hSLP,
                      const char  *pcSrvURL,
                      const char  *pcAttrs,
                      SLPRegReport callback,
@@ -579,7 +595,7 @@ SLPError SLPDelAttrs(SLPHandle   hSLP,
 
 
 /*=========================================================================*/
-SLPError SLPFindSrvTypes(SLPHandle    hSLP,
+SLPEXP SLPError SLPAPI SLPFindSrvTypes(SLPHandle    hSLP,
                          const char  *pcNamingAuthority,
                          const char  *pcScopeList,
                          SLPSrvTypeCallback callback,
@@ -626,7 +642,7 @@ SLPError SLPFindSrvTypes(SLPHandle    hSLP,
 
 
 /*=========================================================================*/
-SLPError SLPFindSrvs(SLPHandle  hSLP,
+SLPEXP SLPError SLPAPI SLPFindSrvs(SLPHandle  hSLP,
                      const char *pcServiceType,
                      const char *pcScopeList,
                      const char *pcSearchFilter,
@@ -672,7 +688,7 @@ SLPError SLPFindSrvs(SLPHandle  hSLP,
 
 
 /*=========================================================================*/
-SLPError SLPFindAttrs(SLPHandle   hSLP,
+SLPEXP SLPError SLPAPI SLPFindAttrs(SLPHandle   hSLP,
                       const char *pcURLOrServiceType,
                       const char *pcScopeList,
                       const char *pcAttrIds,
@@ -722,7 +738,7 @@ SLPError SLPFindAttrs(SLPHandle   hSLP,
 
 
 /*=========================================================================*/
-unsigned short SLPGetRefreshInterval(); 
+SLPEXP unsigned short SLPAPI SLPGetRefreshInterval();
 /*                                                                         */
 /* Returns the maximum across all DAs of the min-refresh-interval          */
 /* attribute.  This value satisfies the advertised refresh interval        */
@@ -739,7 +755,7 @@ unsigned short SLPGetRefreshInterval();
 
 
 /*=========================================================================*/
-SLPError SLPFindScopes(SLPHandle hSLP,
+SLPEXP SLPError SLPAPI SLPFindScopes(SLPHandle hSLP,
                        char** ppcScopeList);
 /*                                                                         */
 /* Sets ppcScopeList parameter to a pointer to a comma separated list      */
@@ -764,21 +780,20 @@ SLPError SLPFindScopes(SLPHandle hSLP,
 
 
 /*=========================================================================*/
-SLPError SLPParseSrvURL(const char *pcSrvURL,
+SLPEXP SLPError SLPAPI SLPParseSrvURL(const char *pcSrvURL,
                         SLPSrvURL** ppSrvURL);
 /*                                                                         */
 /* Parses the URL passed in as the argument into a service URL structure   */
 /* and returns it in the ppSrvURL pointer.  If a parse error occurs,       */
-/* returns SLP_PARSE_ERROR. The input buffer pcSrvURL is destructively     */
-/* modified during the parse and used to fill in the fields of the         */
-/* return structure.  The structure returned in ppSrvURL should be freed   */
-/* with SLPFreeURL().  If the URL has no service part, the s_pcSrvPart     */
-/* string is the empty string, "", i.e.  not NULL. If pcSrvURL is not a    */
-/* service:  URL, then the s_pcSrvType field in the returned data          */
-/* structure is the URL's scheme, which might not be the same as the       */
-/* service type under which the URL was registered.  If the transport is   */
-/* IP, the s_pcTransport field is the empty string.  If the transport is   */
-/* not IP or there is no port number, the s_iPort field is zero.           */
+/* returns SLP_PARSE_ERROR.  The structure returned in ppSrvURL should be  */
+/* freed with SLPFreeURL().  If the URL has no service part, the           */
+/* s_pcSrvPart  string is the empty string, "", i.e.  not NULL. If         */
+/* pcSrvURL is not a service:  URL, then the s_pcSrvType field in the      */
+/* returned data structure is the URL's scheme, which might not be the     */
+/* same as the service type under which the URL was registered.  If the    */
+/* transport is IP, the s_pcTransport field is the empty string.  If the   */
+/* transport is not IP or there is no port number, the s_iPort field is    */
+/* zero.                                                                   */
 /*                                                                         */
 /* pcSrvURL A pointer to a character buffer containing the null terminated */
 /*          URL string to parse.                                           */
@@ -794,7 +809,7 @@ SLPError SLPParseSrvURL(const char *pcSrvURL,
 
 
 /*=========================================================================*/
-SLPError SLPEscape(const char* pcInbuf,
+SLPEXP SLPError SLPAPI SLPEscape(const char* pcInbuf,
                    char** ppcOutBuf,
                    SLPBoolean isTag); 
 /*                                                                         */
@@ -824,7 +839,7 @@ SLPError SLPEscape(const char* pcInbuf,
 
 
 /*=========================================================================*/
-SLPError SLPUnescape(const char* pcInbuf,
+SLPEXP SLPError SLPAPI SLPUnescape(const char* pcInbuf,
                      char** ppcOutBuf,
                      SLPBoolean isTag);
 /*                                                                         */
@@ -854,7 +869,7 @@ SLPError SLPUnescape(const char* pcInbuf,
 
 
 /*=========================================================================*/
-void SLPFree(void* pvMem);                                                 
+SLPEXP void SLPAPI SLPFree(void* pvMem);
 /*                                                                         */
 /* Frees memory returned from SLPParseSrvURL(), SLPFindScopes(),           */
 /* SLPEscape(), and SLPUnescape().                                         */
@@ -866,7 +881,7 @@ void SLPFree(void* pvMem);
 
 
 /*=========================================================================*/
-const char* SLPGetProperty(const char* pcName); 
+SLPEXP const char* SLPAPI SLPGetProperty(const char* pcName);
 /*                                                                         */
 /* Returns the value of the corresponding SLP property name.  The returned */
 /* string is owned by the library and MUST NOT be freed.                   */
@@ -882,7 +897,7 @@ const char* SLPGetProperty(const char* pcName);
 
 
 /*=========================================================================*/
-void SLPSetProperty(const char *pcName,
+SLPEXP void SLPAPI SLPSetProperty(const char *pcName,
                     const char *pcValue);
 /*                                                                         */
 /* Sets the value of the SLP property to the new value.  The pcValue       */
@@ -897,7 +912,7 @@ void SLPSetProperty(const char *pcName,
 
 
 /*=========================================================================*/
-SLPError SLPParseAttrs(const char* pcAttrList,
+SLPEXP SLPError SLPAPI SLPParseAttrs(const char* pcAttrList,
                        const char *pcAttrId,
                        char** ppcAttrVal);
 /*                                                                         */
