@@ -56,6 +56,7 @@
 #include "slpd_regfile.h"
 #include "slpd_property.h"
 #include "slpd_log.h"
+#include "slpd_knownda.h"
 #ifdef ENABLE_PREDICATES
     #include "slpd_predicate.h"
 #endif
@@ -66,6 +67,8 @@
 /*=========================================================================*/
 #include "slp_compare.h"
 #include "slp_xmalloc.h"
+#include "slp_pid.h"
+
 
 /*=========================================================================*/
 SLPDDatabase G_SlpdDatabase;
@@ -134,7 +137,6 @@ void SLPDDatabaseAge(int seconds, int ageall)
     }
 }
 
-
 /*=========================================================================*/
 int SLPDDatabaseReg(SLPMessage msg, SLPBuffer buf)
 /* Add a service registration to the database                              */
