@@ -156,7 +156,7 @@ void IncomingStreamWrite(SLPList* socklist, SLPDSocket* sock)
         }
         else
         {
-#ifdef WIN32
+#ifdef _WIN32
             if (WSAEWOULDBLOCK == WSAGetLastError())
 #else
             if (errno == EWOULDBLOCK)
@@ -189,51 +189,30 @@ void IncomingStreamRead(SLPList* socklist, SLPDSocket* sock)
                              MSG_PEEK,
                              (struct sockaddr *)&(sock->peeraddr),
                              &peeraddrlen);
-        if (bytesread <= 0)
+        if (bytesread > 0)
         {
-            sock->state = SOCKET_CLOSE;
-            return;
-        }
 
-        if (*peek == 2)
-        {
-            recvlen = AsUINT24(peek + 2);
-        }
-        else if (*peek == 1) /* SLPv1 packet */
-        {
-            recvlen = AsUINT16(peek + 2);
+            if (*peek == 2)
+                recvlen = AsUINT24(peek + 2);
+            else if (*peek == 1) /* SLPv1 packet */
+                recvlen = AsUINT16(peek + 2);
+            /* allocate the recvbuf big enough for the whole message */
+            sock->recvbuf = SLPBufferRealloc(sock->recvbuf,recvlen);
+            if (sock->recvbuf)
+            {
+                sock->state = STREAM_READ; 
+            }
+            else
+            {
+                SLPDLog("INTERNAL_ERROR - out of memory!\n");
+                sock->state = SOCKET_CLOSE;
+            }
         }
         else
         {
-            /* Version not supported */
-            SLPDLog("WARNING - Version %i not supported from %s",
-                    *peek,    
-                    inet_ntoa(sock->peeraddr.sin_addr));
             sock->state = SOCKET_CLOSE;
             return;
-        }
-        
-        /* Check the recvlen for a sane value */
-        if(recvlen > 0xffff)
-        {
-            SLPDLog("WARNING - Message too large. %i bytes from %s",
-                    recvlen,    
-                    inet_ntoa(sock->peeraddr.sin_addr));
-            sock->state = SOCKET_CLOSE;
-            return;
-        }
-
-        /* allocate the recvbuf big enough for the whole message */
-        sock->recvbuf = SLPBufferRealloc(sock->recvbuf,recvlen);
-        if (sock->recvbuf)
-        {
-            sock->state = STREAM_READ; 
-        }
-        else
-        {
-            SLPDLog("ERROR - out of memory\n");
-            sock->state = SOCKET_CLOSE;
-        }
+        }        
     }
 
     if (sock->state == STREAM_READ)
@@ -286,7 +265,7 @@ void IncomingSocketListen(SLPList* socklist, SLPDSocket* sock)
     SLPDSocket*         connsock;
     struct sockaddr_in  peeraddr;
     socklen_t           peeraddrlen;
-#ifdef WIN32
+#ifdef _WIN32
     const char   lowat = SLPD_SMALLEST_MESSAGE;
 #else    
     const int   lowat = SLPD_SMALLEST_MESSAGE;
@@ -315,7 +294,7 @@ void IncomingSocketListen(SLPList* socklist, SLPDSocket* sock)
                 setsockopt(connsock->fd,SOL_SOCKET,SO_RCVLOWAT,&lowat,sizeof(lowat));
                 setsockopt(connsock->fd,SOL_SOCKET,SO_SNDLOWAT,&lowat,sizeof(lowat)); 
                 /* set accepted socket to non blocking */
-#ifdef WIN32
+#ifdef _WIN32
                 fdflags = 1;
                 ioctlsocket(connsock->fd, FIONBIO, &fdflags);
 #else
