@@ -69,6 +69,9 @@
 #include "slp_utf8.h"
 #include "slp_compare.h"
 #include "slp_xid.h"
+#include "slp_dhcp.h"
+#include "slp_parse.h"
+#include "slp_net.h"
 #ifdef ENABLE_SLPv2_SECURITY
     #include "slp_auth.h"
     #include "slp_spi.h"
@@ -86,11 +89,12 @@ SLPDatabase G_SlpdKnownDAs;
 int G_KnownDATimeSinceLastRefresh = 0;
 /*=========================================================================*/
 
-
 /*-------------------------------------------------------------------------*/
 int MakeActiveDiscoveryRqst(int ismcast, SLPBuffer* buffer)
-/* Pack a buffer with service:directory-agent SrvRqst                      */
-/*-------------------------------------------------------------------------*/
+/* Pack a buffer with service:directory-agent SrvRqst                      *
+ *                                                                         *
+ * Caller must free buffer                                                 *
+ *-------------------------------------------------------------------------*/
 {
     size_t          size;
     void*           eh;
@@ -100,7 +104,7 @@ int MakeActiveDiscoveryRqst(int ismcast, SLPBuffer* buffer)
     int             errorcode   = 0;
     SLPBuffer       tmp         = 0;
     SLPBuffer       result      = *buffer;
-   
+
     /*-------------------------------------------------*/
     /* Generate a DA service request buffer to be sent */
     /*-------------------------------------------------*/
@@ -299,6 +303,115 @@ void SLPDKnownDARegisterAll(SLPMessage daadvert, int immortalonly)
 }  
 
 
+/*-------------------------------------------------------------------------*/
+int MakeSrvderegFromSrvReg(SLPMessage msg, 
+                           SLPBuffer inbuf,
+                           SLPBuffer* outbuf)
+/* Pack a buffer with a SrvDereg message using information from an existing
+ * SrvReg message
+ * 
+ * Caller must free outbuf
+ *-------------------------------------------------------------------------*/
+{
+    int                 size;
+    SLPBuffer           sendbuf;
+    SLPSrvReg*          srvreg;
+    
+    srvreg = &(msg->body.srvreg);
+
+    /*-------------------------------------------------------------*/
+    /* ensure the buffer is big enough to handle the whole srvdereg*/
+    /*-------------------------------------------------------------*/
+    size = msg->header.langtaglen + 18; /* 14 bytes for header     */
+                                           /*  2 bytes for scopelen */
+                                           /*  see below for URLEntry */
+                                           /*  2 bytes for taglist len */
+    if ( srvreg->urlentry.opaque )
+    {
+        size += srvreg->urlentry.opaquelen;
+    }
+    else
+    {
+        size += 6; /* +6 for the static portion of the url-entry */
+        size += srvreg->urlentry.urllen;
+    }
+    size += srvreg->scopelistlen;
+    /* taglistlen is always 0 */
+
+    *outbuf = sendbuf = SLPBufferAlloc(size);
+    if (*outbuf == NULL)
+    {
+        return SLP_ERROR_INTERNAL_ERROR;
+    }
+
+    /*----------------------*/
+    /* Construct a SrvDereg */
+    /*----------------------*/
+    /*version*/
+    *(sendbuf->start)       = 2;
+    /*function id*/
+    *(sendbuf->start + 1)   = SLP_FUNCT_SRVDEREG;
+    /*length*/
+    ToUINT24(sendbuf->start + 2, size);
+    /*flags*/
+    ToUINT16(sendbuf->start + 5,
+             (size > SLP_MAX_DATAGRAM_SIZE ? SLP_FLAG_OVERFLOW : 0));
+    /*ext offset*/
+    ToUINT24(sendbuf->start + 7,0);
+    /*xid*/
+    ToUINT16(sendbuf->start + 10,SLPXidGenerate());
+    /*lang tag len*/
+    ToUINT16(sendbuf->start + 12,msg->header.langtaglen);
+    /*lang tag*/
+    memcpy(sendbuf->start + 14,
+           msg->header.langtag,
+           msg->header.langtaglen);
+    sendbuf->curpos = sendbuf->start + 14 + msg->header.langtaglen;
+
+    /* scope list */
+    ToUINT16(sendbuf->curpos, srvreg->scopelistlen);
+    sendbuf->curpos = sendbuf->curpos + 2;
+    memcpy(sendbuf->curpos,srvreg->scopelist,srvreg->scopelistlen);
+    sendbuf->curpos = sendbuf->curpos + srvreg->scopelistlen;
+    /* the urlentry */
+#ifdef ENABLE_SLPv1
+    if ( srvreg->urlentry.opaque == 0 )
+    {
+        /* url-entry reserved */
+        *sendbuf->curpos = 0;        
+        sendbuf->curpos += 1;
+        /* url-entry lifetime */
+        ToUINT16(sendbuf->curpos,srvreg->urlentry.lifetime);
+        sendbuf->curpos = sendbuf->curpos + 2;
+        /* url-entry urllen */
+        ToUINT16(sendbuf->curpos,srvreg->urlentry.urllen);
+        sendbuf->curpos += 2;
+        /* url-entry url */
+        memcpy(sendbuf->curpos,
+               srvreg->urlentry.url,
+               srvreg->urlentry.urllen);
+        sendbuf->curpos += srvreg->urlentry.urllen;                        
+        /* url-entry authcount */
+        *sendbuf->curpos = 0;        
+        sendbuf->curpos += 1;
+    }
+    else
+#endif /* ENABLE_SLPv1 */
+    {
+        memcpy(sendbuf->curpos,
+               srvreg->urlentry.opaque,
+               srvreg->urlentry.opaquelen);
+        sendbuf->curpos += srvreg->urlentry.opaquelen;
+    }
+
+    /* taglist (always 0) */
+    ToUINT16(sendbuf->curpos,0);
+    sendbuf->curpos += 1;
+
+    return 0;
+}
+
+
 /*-------------------------------------------------------------------------*/
 void SLPDKnownDADeregisterAll(SLPMessage daadvert)
 /* de-registers all services with specified DA                             */
@@ -308,7 +421,6 @@ void SLPDKnownDADeregisterAll(SLPMessage daadvert)
     SLPMessage          msg;
     SLPSrvReg*          srvreg;
     SLPDSocket*         sock;
-    size_t              size;
     SLPBuffer           sendbuf     = 0;
     void*               handle      = 0;
 
@@ -342,91 +454,8 @@ void SLPDKnownDADeregisterAll(SLPMessage daadvert)
             if ( srvreg->source == SLP_REG_SOURCE_LOCAL || 
                  srvreg->source == SLP_REG_SOURCE_STATIC )
             {
-                /*-------------------------------------------------------------*/
-                /* ensure the buffer is big enough to handle the whole srvdereg*/
-                /*-------------------------------------------------------------*/
-                size = msg->header.langtaglen + 18; /* 14 bytes for header     */
-                                                    /*  2 bytes for scopelen */
-                                                    /*  see below for URLEntry */
-                                                    /*  2 bytes for taglist len */
-                if ( srvreg->urlentry.opaque )
+                if(MakeSrvderegFromSrvReg(msg,buf,&sendbuf) == 0)
                 {
-                    size += srvreg->urlentry.opaquelen;
-                }
-                else
-                {
-                    size += 6; /* +6 for the static portion of the url-entry */
-		    size += srvreg->urlentry.urllen;
-                }
-                size += srvreg->scopelistlen;
-                /* taglistlen is always 0 */
-
-                sendbuf = SLPBufferAlloc(size);
-                if ( sendbuf )
-                {
-                    /*----------------------*/
-                    /* Construct a SrvDereg */
-                    /*----------------------*/
-                    /*version*/
-                    *(sendbuf->start)       = 2;
-                    /*function id*/
-                    *(sendbuf->start + 1)   = SLP_FUNCT_SRVDEREG;
-                    /*length*/
-                    ToUINT24(sendbuf->start + 2, size);
-                    /*flags*/
-                    ToUINT16(sendbuf->start + 5,
-                             (size > SLP_MAX_DATAGRAM_SIZE ? SLP_FLAG_OVERFLOW : 0));
-                    /*ext offset*/
-                    ToUINT24(sendbuf->start + 7,0);
-                    /*xid*/
-                    ToUINT16(sendbuf->start + 10,SLPXidGenerate());
-                    /*lang tag len*/
-                    ToUINT16(sendbuf->start + 12,msg->header.langtaglen);
-                    /*lang tag*/
-                    memcpy(sendbuf->start + 14,
-                           msg->header.langtag,
-                           msg->header.langtaglen);
-                    sendbuf->curpos = sendbuf->start + 14 + msg->header.langtaglen;
-
-                    /* scope list */
-                    ToUINT16(sendbuf->curpos, srvreg->scopelistlen);
-                    sendbuf->curpos = sendbuf->curpos + 2;
-                    memcpy(sendbuf->curpos,srvreg->scopelist,srvreg->scopelistlen);
-                    sendbuf->curpos = sendbuf->curpos + srvreg->scopelistlen;
-                    /* the urlentry */
-#ifdef ENABLE_SLPv1
-                    if ( srvreg->urlentry.opaque == 0 )
-                    {
-                        /* url-entry reserved */
-                        *sendbuf->curpos = 0;        
-                        sendbuf->curpos = sendbuf->curpos + 1;
-                        /* url-entry lifetime */
-                        ToUINT16(sendbuf->curpos,srvreg->urlentry.lifetime);
-                        sendbuf->curpos = sendbuf->curpos + 2;
-                        /* url-entry urllen */
-                        ToUINT16(sendbuf->curpos,srvreg->urlentry.urllen);
-                        sendbuf->curpos = sendbuf->curpos + 2;
-                        /* url-entry url */
-                        memcpy(sendbuf->curpos,
-                               srvreg->urlentry.url,
-                               srvreg->urlentry.urllen);
-                        sendbuf->curpos = sendbuf->curpos + srvreg->urlentry.urllen;                        
-                        /* url-entry authcount */
-                        *sendbuf->curpos = 0;        
-                        sendbuf->curpos = sendbuf->curpos + 1;
-                    }
-                    else
-#endif /* ENABLE_SLPv1 */
-                    {
-                        memcpy(sendbuf->curpos,
-                               srvreg->urlentry.opaque,
-                               srvreg->urlentry.opaquelen);
-                        sendbuf->curpos = sendbuf->curpos + srvreg->urlentry.opaquelen;
-                    }
-
-                    /* taglist (always 0) */
-                    ToUINT16(sendbuf->curpos,0);
-
                     /*--------------------------------------------------*/
                     /* link newly constructed buffer to socket sendlist */
                     /*--------------------------------------------------*/
@@ -445,22 +474,127 @@ void SLPDKnownDADeregisterAll(SLPMessage daadvert)
 
 
 /*=========================================================================*/
-int SLPDKnownDAInit()
-/* Initializes the KnownDA list.  Removes all entries and adds entries     */
-/* that are statically configured.                                         */
+int SLPDKnownDAFromDHCP()
+/* Queries DHCP for configured DA's.													*/
 /*                                                                         */
 /* returns  zero on success, Non-zero on failure                           */
 /*=========================================================================*/
 {
-    char*               temp;
-    char*               tempend;
-    char*               slider1;
-    char*               slider2;
-    struct hostent*     he;
-    struct in_addr      daaddr;
-    SLPDSocket*         sock;
-    SLPBuffer           buf;
+	SLPBuffer			buf;
+	DHCPContext			ctx;
+	SLPDSocket*			sock;
+	struct in_addr		daaddr;
+	unsigned char *	alp;
+	unsigned char		dhcpOpts[] = {TAG_SLP_SCOPE, TAG_SLP_DA};
 
+	*ctx.scopelist = 0;
+	ctx.addrlistlen = 0;
+
+	DHCPGetOptionInfo(dhcpOpts, sizeof(dhcpOpts), DHCPParseSLPTags, &ctx);
+
+	alp = ctx.addrlist;
+	while(ctx.addrlistlen >= 4)
+	{
+		memcpy(&daaddr.s_addr, alp, 4);
+		if (daaddr.s_addr)
+		{
+			/*--------------------------------------------------------
+				Get an outgoing socket to the DA and set it up to make
+				the service:directoryagent request
+			  --------------------------------------------------------*/
+			sock = SLPDOutgoingConnect(&daaddr);
+			if (sock)
+			{
+			   buf = 0;
+			   if (MakeActiveDiscoveryRqst(0,&buf) == 0)
+			   {
+					if (sock->state == STREAM_CONNECT_IDLE)
+						sock->state = STREAM_WRITE_FIRST;
+					SLPListLinkTail(&(sock->sendlist),(SLPListItem*)buf);
+					if (sock->state == STREAM_CONNECT_IDLE)
+						sock->state = STREAM_WRITE_FIRST;
+			   }
+			}
+		}
+		ctx.addrlistlen -= 4;
+		alp += 4;
+	}
+	return 0;
+}
+
+/*=========================================================================*/
+int SLPKnownDAFromProperties()
+/* Queries static configuration for DA's.												*/
+/*                                                                         */
+/* returns  zero on success, Non-zero on failure                           */
+/*=========================================================================*/
+{
+	char*               temp;
+	char*               tempend;
+	char*               slider1;
+	char*               slider2;
+	struct hostent*     he;
+	struct in_addr      daaddr;
+	SLPDSocket*         sock;
+	SLPBuffer           buf;
+
+	if (G_SlpdProperty.DAAddresses && *G_SlpdProperty.DAAddresses)
+	{
+		temp = slider1 = xstrdup(G_SlpdProperty.DAAddresses);
+		if (temp)
+		{
+			tempend = temp + strlen(temp);
+			while (slider1 < tempend)
+			{
+				while (*slider1 && *slider1 == ' ') slider1++;
+				slider2 = slider1;
+				while (*slider2 && *slider2 != ',') slider2++;
+				*slider2++ = 0;
+
+				daaddr.s_addr = 0;
+				if(inet_aton(slider1, &daaddr) == 0)
+				{
+					he = gethostbyname(slider1);
+					if (he)
+						daaddr.s_addr = *((unsigned int*)(he->h_addr_list[0]));
+				}
+			    
+				if(daaddr.s_addr)
+				{
+					/*--------------------------------------------------------*/
+					/* Get an outgoing socket to the DA and set it up to make */
+					/* the service:directoryagent request                     */
+					/*--------------------------------------------------------*/
+					sock = SLPDOutgoingConnect(&daaddr);
+					if (sock)
+					{
+						buf = 0;
+						if (MakeActiveDiscoveryRqst(0,&buf) == 0)
+						{
+							if (sock->state == STREAM_CONNECT_IDLE)
+								sock->state = STREAM_WRITE_FIRST;
+							SLPListLinkTail(&(sock->sendlist),(SLPListItem*)buf);
+							if (sock->state == STREAM_CONNECT_IDLE)
+								sock->state = STREAM_WRITE_FIRST;
+						}
+					}
+				}
+				slider1 = slider2;
+			}
+			xfree(temp);
+		}
+	}
+	return 0;
+}
+
+/*=========================================================================*/
+int SLPDKnownDAInit()
+/* Initializes the KnownDA list.  Removes all entries and adds entries     */
+/* that are statically configured.  Adds entries configured through DHCP.  */
+/*                                                                         */
+/* returns  zero on success, Non-zero on failure                           */
+/*=========================================================================*/
+{
     /*--------------------------------------*/
     /* Set initialize the DAAdvert database */
     /*--------------------------------------*/
@@ -470,68 +604,18 @@ int SLPDKnownDAInit()
     /* Added statically configured DAs to the Known DA List by sending */
     /* active DA discovery requests directly to them                   */
     /*-----------------------------------------------------------------*/
-    if ( G_SlpdProperty.DAAddresses && *G_SlpdProperty.DAAddresses )
-    {
-        temp = slider1 = xstrdup(G_SlpdProperty.DAAddresses);
-        if ( temp )
-        {
-            tempend = temp + strlen(temp);
-            while ( slider1 < tempend )
-            {
-                while ( *slider1 && *slider1 == ' ' ) slider1++;
-                slider2 = slider1;
-                while ( *slider2 && *slider2 != ',' ) slider2++;
-                *slider2++ = 0;
-
-                daaddr.s_addr = 0;
-                if(inet_aton(slider1, &daaddr) == 0)
-                {
-                    he = gethostbyname(slider1);
-                    if (he)
-                    {
-                        daaddr.s_addr = *((unsigned int*)(he->h_addr_list[0]));
-                    }
-                }
-                
-                if(daaddr.s_addr)
-                {
-                    /*--------------------------------------------------------*/
-                    /* Get an outgoing socket to the DA and set it up to make */
-                    /* the service:directoryagent request                     */
-                    /*--------------------------------------------------------*/
-                    sock = SLPDOutgoingConnect(&daaddr);
-                    if ( sock )
-                    {
-                        buf = 0;
-                        if ( MakeActiveDiscoveryRqst(0,&buf) == 0 )
-                        {
-                            if ( sock->state == STREAM_CONNECT_IDLE )
-                            {
-                                sock->state = STREAM_WRITE_FIRST;
-                            }
-                            SLPListLinkTail(&(sock->sendlist),(SLPListItem*)buf);
-                            if ( sock->state == STREAM_CONNECT_IDLE )
-                            {
-                                sock->state = STREAM_WRITE_FIRST;
-                            }
-                        }
-                    }
-                }
-
-                slider1 = slider2;
-            }
-
-            xfree(temp);
-        }
-    }
+    SLPKnownDAFromProperties();
 
+    /*-----------------------------------------------------------------*/
+    /* Discover DHCP DA's and add them to the active discovery list.	  */
+    /*-----------------------------------------------------------------*/
+    SLPDKnownDAFromDHCP();
 
     /*----------------------------------------*/
     /* Lastly, Perform first active discovery */
     /*----------------------------------------*/
     SLPDKnownDAActiveDiscovery(0);
 
-
     return 0;
 }
 
@@ -583,6 +667,8 @@ int SLPDKnownDAAdd(SLPMessage msg, SLPBuffer buf)
     SLPDatabaseEntry*   entry;
     SLPDAAdvert*        entrydaadvert;
     SLPDAAdvert*        daadvert;
+    struct in_addr      daaddr;
+    SLPParsedSrvUrl*    parsedurl       = NULL;
     int                 result          = 0;
     SLPDatabaseHandle   dh              = NULL;
 
@@ -596,6 +682,35 @@ int SLPDKnownDAAdd(SLPMessage msg, SLPBuffer buf)
     /* daadvert is the DAAdvert message being added */
     daadvert = &(msg->body.daadvert);
 
+    
+    /* --------------------------------------------------------
+     * Make sure that the peer address in the DAAdvert matches
+     * the host in the DA service URL.
+     *---------------------------------------------------------
+     */
+    if (SLPParseSrvUrl(daadvert->urllen,
+                       daadvert->url,
+                       &parsedurl))
+    {
+        /* could not parse the DA service url */
+        result = SLP_ERROR_PARSE_ERROR;
+        goto CLEANUP;
+    }
+    if (SLPNetResolveHostToAddr(parsedurl->host,&daaddr))
+    {
+        /* Unable to resolve the host in the DA advert to an address */
+        xfree(parsedurl);
+        result = SLP_ERROR_PARSE_ERROR;
+        goto CLEANUP;
+    }
+    /* free the parsed url created in call to SLPParseSrvUrl() */
+    xfree(parsedurl);
+    /* set the peer address in the DAAdvert message so that it matches
+     * the address the DA service URL resolves to 
+     */
+    msg->peer.sin_addr = daaddr;
+
+
     /*-----------------------------------------------------*/
     /* Check to see if there is already an identical entry */
     /*-----------------------------------------------------*/
@@ -614,7 +729,7 @@ int SLPDKnownDAAdd(SLPMessage msg, SLPBuffer buf)
                               daadvert->url) == 0 )
         {
 
-            #ifdef ENABLE_SLPv2_SECURITY                
+#ifdef ENABLE_SLPv2_SECURITY                
             if ( G_SlpdProperty.checkSourceAddr &&
                  memcmp(&(entry->msg->peer.sin_addr),
                         &(msg->peer.sin_addr),
@@ -633,9 +748,10 @@ int SLPDKnownDAAdd(SLPMessage msg, SLPBuffer buf)
                 SLPDatabaseClose(dh);
                 result = SLP_ERROR_AUTHENTICATION_FAILED;
                 goto CLEANUP;
-            }
-            #endif
-
+            } 
+            
+#endif
+            
             if ( daadvert->bootstamp != 0 &&
                  daadvert->bootstamp <= entrydaadvert->bootstamp )
             {
@@ -650,52 +766,73 @@ int SLPDKnownDAAdd(SLPMessage msg, SLPBuffer buf)
         }
     }
 
-    if ( entry == 0 )
-    {
-        /* Advertising DA is new to us */
-        SLPDKnownDARegisterAll(msg,0);
-    }
-    else
-    {
-        /* The advertising DA is not new to us, but the old entry   */
-        /* has been deleted from our database so that tne new entry */
-        /* with its up to date time stamp can be put back in        */
-    }
-
     /* Make sure the DA is not dying */
-    if ( daadvert->bootstamp != 0 )
+    if (daadvert->bootstamp != 0)
     {
-        /* Create and link in a new entry */
-        entry = SLPDatabaseEntryCreate(msg,buf);
-        if ( entry )
+        if ( entry == 0 )
         {
-            SLPDatabaseAdd(dh, entry);
-            SLPDLogDAAdvertisement("Addition",entry);
+            /* create a new database entry using the DAAdvert message */
+            entry = SLPDatabaseEntryCreate(msg,buf);
+            if (entry)
+            {
+                SLPDatabaseAdd(dh, entry);
+
+                /* register all the services we know about with this new DA */
+                SLPDKnownDARegisterAll(msg,0);
+
+                /* log the addition of a new DA */
+                SLPDLogDAAdvertisement("Addition",entry);
+            }
+            else
+            {
+                /* Could not create a new entry */
+                result = SLP_ERROR_INTERNAL_ERROR;
+                goto CLEANUP;
+            }
         }
         else
         {
-            result = SLP_ERROR_INTERNAL_ERROR;
-            goto CLEANUP;
+            /* The advertising DA is not new to us, but the old entry   */
+            /* has been deleted from our database so that the new entry */
+            /* with its up to date time stamp can be put back in.       */
+            /* create a new database entry using the DAAdvert message */
+            entry = SLPDatabaseEntryCreate(msg,buf);
+            if (entry)
+            {
+                SLPDatabaseAdd(dh, entry);
+            }
+            else
+            {
+                /* Could not create a new entry */
+                result = SLP_ERROR_INTERNAL_ERROR;
+                goto CLEANUP;
+            }
         }
+
+        SLPDatabaseClose(dh);
+
+        return result;
     }
     else
     {
-        /* Dying DAs are not recorded in our database */
-        goto CLEANUP;
+        /* DA is dying */
+        if (entry)
+        {
+            /* Dying DA was found in our KnownDA database. Log that it
+             * was removed.
+             */
+            SLPDLogDAAdvertisement("Removed",entry);
+        }
     }
 
-    SLPDatabaseClose(dh);
-
-    return result;
-
-CLEANUP:
+    CLEANUP:
     /* If we are here, we need to cleanup the message descriptor and the  */
     /* message buffer because they were not added to the database and not */
     /* cleaning them up would result in a memory leak                     */
     /* We also need to make sure the Database handle is closed.           */
     SLPMessageFree(msg);
     SLPBufferFree(buf);
-    if(dh) SLPDatabaseClose(dh);
+    if (dh) SLPDatabaseClose(dh);
 
     return result;
 }
@@ -722,8 +859,8 @@ void SLPDKnownDARemove(struct in_addr* addr)
             /* Assume DAs are identical if their peer match */
             if ( memcmp(addr,&(entry->msg->peer.sin_addr),sizeof(*addr)) == 0 )
             {
-                SLPDLogDAAdvertisement("Removal",entry);
                 SLPDatabaseRemove(dh,entry);
+                SLPDLogDAAdvertisement("Removal",entry);
                 break;            
             }
         }
@@ -854,6 +991,7 @@ int SLPDKnownDAGenerateMyDAAdvert(int errorcode,
 
 
 
+
     /*-------------------------------------------------------------*/
     /* ensure the buffer is big enough to handle the whole srvrply */
     /*-------------------------------------------------------------*/
@@ -956,6 +1094,7 @@ int SLPDKnownDAGenerateMyDAAdvert(int errorcode,
 #endif
 
 
+
         /* authblock count */
 #ifdef ENABLE_SLPv2_SECURITY
         if ( daadvertauth )
@@ -1241,7 +1380,7 @@ void SLPDKnownDAActiveDiscovery(int seconds)
         return;
     }
     /* When activeDiscoveryXmits is < 0 then we should not xmit any more */
-    if(G_SlpdProperty.activeDiscoveryXmits < 0)
+    if (G_SlpdProperty.activeDiscoveryXmits < 0)
     {
         return ;
     }
@@ -1314,12 +1453,13 @@ void SLPDKnownDAPassiveDAAdvert(int seconds, int dadead)
     SLPDSocket*     v1sock;
 #endif
 
+
     /* SAs don't send passive DAAdverts */
-    if( G_SlpdProperty.isDA == 0)
+    if ( G_SlpdProperty.isDA == 0)
     {
         return;
     }
-    
+
     /* Check to see if we should perform passive DA detection */
     if ( G_SlpdProperty.passiveDADetection == 0 )
     {
@@ -1328,7 +1468,7 @@ void SLPDKnownDAPassiveDAAdvert(int seconds, int dadead)
 
     if ( G_SlpdProperty.nextPassiveDAAdvert <= 0 || dadead )
     {
-        G_SlpdProperty.nextPassiveDAAdvert = SLPD_CONFIG_DA_BEAT;
+        G_SlpdProperty.nextPassiveDAAdvert = G_SlpdProperty.DAHeartBeat;
 
         /*--------------------------------------------------*/
         /* Create new DATAGRAM socket with appropriate peer */
@@ -1337,8 +1477,8 @@ void SLPDKnownDAPassiveDAAdvert(int seconds, int dadead)
         {
             peeraddr.s_addr = htonl(SLP_MCAST_ADDRESS);
             sock = SLPDSocketCreateDatagram(&peeraddr,DATAGRAM_MULTICAST);
-            
-            #ifdef ENABLE_SLPv1
+
+#ifdef ENABLE_SLPv1
             if ( !dadead )
             {
                 peeraddr.s_addr = htonl(SLPv1_DA_MCAST_ADDRESS);
@@ -1349,14 +1489,14 @@ void SLPDKnownDAPassiveDAAdvert(int seconds, int dadead)
             {
                 v1sock = NULL;
             }
-            #endif  
+#endif  
         }
         else
         {
             peeraddr.s_addr = htonl(SLP_BCAST_ADDRESS);
             sock = SLPDSocketCreateDatagram(&peeraddr,DATAGRAM_BROADCAST);
-            
-            #ifdef ENABLE_SLPv1
+
+#ifdef ENABLE_SLPv1
             if ( !dadead )
             {
                 v1sock = SLPDSocketCreateDatagram(&peeraddr,DATAGRAM_BROADCAST);
@@ -1365,14 +1505,14 @@ void SLPDKnownDAPassiveDAAdvert(int seconds, int dadead)
             {
                 v1sock = NULL;
             }
-            #endif
+#endif
         }
 
         /* Generate the DAAdvert and link it to the write list */
         if ( sock )
         {
-            if(SLPDKnownDAGenerateMyDAAdvert(0,dadead,0,&(sock->sendbuf)) == 0)
-            {   
+            if (SLPDKnownDAGenerateMyDAAdvert(0,dadead,0,&(sock->sendbuf)) == 0)
+            {
                 SLPDOutgoingDatagramWrite(sock);
             }
             else
@@ -1381,16 +1521,16 @@ void SLPDKnownDAPassiveDAAdvert(int seconds, int dadead)
             }
         }
 
-        #ifdef ENABLE_SLPv1
+#ifdef ENABLE_SLPv1
         if ( v1sock )
         {
             /* SLPv1 does not support shutdown messages */
-            
+
             /* Generate the DAAdvert and write it */
-            if(SLPDKnownDAGenerateMyV1DAAdvert(0,
-                                               SLP_CHAR_UTF8,
-                                               SLPXidGenerate(),
-                                               &(v1sock->sendbuf)) == 0)
+            if (SLPDKnownDAGenerateMyV1DAAdvert(0,
+                                                SLP_CHAR_UTF8,
+                                                SLPXidGenerate(),
+                                                &(v1sock->sendbuf)) == 0)
             {
                 SLPDOutgoingDatagramWrite(v1sock);
             }
@@ -1399,7 +1539,7 @@ void SLPDKnownDAPassiveDAAdvert(int seconds, int dadead)
                 SLPDSocketFree(v1sock);
             }
         }
-        #endif
+#endif
     }
     else
     {
@@ -1425,7 +1565,7 @@ void SLPDKnownDAImmortalRefresh(int seconds)
     {
         /* Refresh all SLP_LIFETIME_MAXIMUM registrations */
         dh = SLPDatabaseOpen(&G_SlpdKnownDAs);
-        if ( dh )
+        if(dh)
         {
 
             /*-----------------------------------------------------*/
@@ -1457,6 +1597,57 @@ void SLPDKnownDAImmortalRefresh(int seconds)
 }
 
 
+/*=========================================================================*/
+void SLPDKnownDADeRegisterWithAllDas(SLPMessage msg, SLPBuffer buf)
+/* Deregister the registration described by the specified message          */
+/*                                                                         */
+/* msg (IN) A message descriptor for a SrvReg or SrvDereg message to       */
+/*          deregister                                                     */
+/*                                                                         */
+/* buf (IN) Message buffer associated with msg                             */
+/*                                                                         */
+/* Returns: None                                                           */
+/*=========================================================================*/
+{
+    SLPBuffer  sendbuf;
+    
+    if(msg->header.functionid == SLP_FUNCT_SRVREG)
+    {
+        if(MakeSrvderegFromSrvReg(msg,buf, &sendbuf) == 0)
+        {
+            SLPDKnownDAEcho(msg,sendbuf);
+            SLPBufferFree(sendbuf); 
+        }
+    }
+    else if (msg->header.functionid == SLP_FUNCT_SRVDEREG)
+    {
+        /* Simply echo the message through as is */
+        SLPDKnownDAEcho(msg,buf);
+    }
+}
+
+
+/*=========================================================================*/
+void SLPDKnownDARegisterWithAllDas(SLPMessage msg, SLPBuffer buf)
+/* Register the registration described by the specified message with all   */
+/* known DAs                                                               */
+/*                                                                         */
+/* msg (IN) A message descriptor for a SrvReg or SrvDereg message to       */
+/*          deregister                                                     */
+/*                                                                         */
+/* buf (IN) Message buffer associated with msg                             */
+/*                                                                         */
+/* Returns: None                                                           */
+/*=========================================================================*/
+{
+    if (msg->header.functionid == SLP_FUNCT_SRVDEREG)
+    {
+        /* Simply echo the message through as is */
+        SLPDKnownDAEcho(msg,buf);
+    }
+}
+
+
 #ifdef DEBUG
 /*=========================================================================*/
 void SLPDKnownDADump()
