@@ -263,7 +263,7 @@ void HandleSigInt()
 #endif
 
 
-#ifndef WIN32
+#ifndef _WIN32
 /*-------------------------------------------------------------------------*/
 int CheckPid(const char* pidfile)
 /* Check a pid file to see if slpd is already running                      */
@@ -396,7 +396,7 @@ int Daemonize(const char* pidfile)
     /* Set cwd to / (root)*/
     /*--------------------*/
     chdir("/");
-
+    
     return 0;
 }
 
@@ -633,7 +633,7 @@ int main(int argc, char* argv[])
 
     return 0;
 }
-#endif /*ifndef WIN32 */
+#endif /*ifndef _WIN32 */
 
 
 
