@@ -100,7 +100,7 @@ void OutgoingDatagramRead(SLPList* socklist, SLPDSocket* sock)
 void OutgoingStreamReconnect(SLPList* socklist, SLPDSocket* sock)
 /*-------------------------------------------------------------------------*/
 {
-#ifdef WIN32
+#ifdef _WIN32
     u_long              fdflags;
 #else    
     int                 fdflags;
@@ -152,7 +152,7 @@ void OutgoingStreamReconnect(SLPList* socklist, SLPDSocket* sock)
     /*---------------------------------------------*/
     /* Set the new socket to enable nonblocking IO */
     /*---------------------------------------------*/
-#ifdef WIN32
+#ifdef _WIN32
     fdflags = 1;
     ioctlsocket(sock->fd, FIONBIO, &fdflags);
 #else
@@ -168,7 +168,7 @@ void OutgoingStreamReconnect(SLPList* socklist, SLPDSocket* sock)
                  (struct sockaddr *)&(sock->peeraddr), 
                  sizeof(struct sockaddr_in)) )
     {
-#ifdef WIN32
+#ifdef _WIN32
         if ( WSAEWOULDBLOCK == WSAGetLastError() )
 #else
         if ( errno == EINPROGRESS )
@@ -191,7 +191,6 @@ void OutgoingStreamRead(SLPList* socklist, SLPDSocket* sock)
 /*-------------------------------------------------------------------------*/
 {
     int     bytesread;
-    int     recvlen;
     char    peek[16];
     int     peeraddrlen = sizeof(struct sockaddr_in);
 
@@ -208,52 +207,21 @@ void OutgoingStreamRead(SLPList* socklist, SLPDSocket* sock)
                              &peeraddrlen);
         if ( bytesread > 0 )
         {
-            
-            /* Check the version */
-            if (*peek == 2)
-            {
-                recvlen = AsUINT24(peek + 2);
-            }
-            else if (*peek == 1) /* SLPv1 packet */
-            {
-                recvlen = AsUINT16(peek + 2);
-            }
-            else
-            {
-                /* Version not supported */
-                SLPDLog("WARNING - Version %i not supported from %s",
-                        *peek,    
-                        inet_ntoa(sock->peeraddr.sin_addr));
-                sock->state = SOCKET_CLOSE;
-                return;
-            }
-            
-            /* Check the recvlen for a sane value */
-            recvlen = AsUINT24(peek+2);
-            if(recvlen > 0xffff)
-            {
-                SLPDLog("WARNING - Message too large. %i bytes from %s",
-                        recvlen,    
-                        inet_ntoa(sock->peeraddr.sin_addr));
-                sock->state = SOCKET_CLOSE;
-                return;
-            }
-            
             /* allocate the recvbuf big enough for the whole message */
-            sock->recvbuf = SLPBufferRealloc(sock->recvbuf,recvlen);
+            sock->recvbuf = SLPBufferRealloc(sock->recvbuf,AsUINT24(peek+2));
             if ( sock->recvbuf )
             {
                 sock->state = STREAM_READ;
             }
             else
             {
-                SLPDLog("ERROR - out of memory");
+                SLPDLog("INTERNAL_ERROR - out of memory!\n");
                 sock->state = SOCKET_CLOSE;
             }
         }
         else
         {
-#ifdef WIN32
+#ifdef _WIN32
             if ( WSAEWOULDBLOCK != WSAGetLastError() )
 #else
             if ( errno != EWOULDBLOCK )
@@ -313,7 +281,7 @@ void OutgoingStreamRead(SLPList* socklist, SLPDSocket* sock)
         }
         else
         {
-#ifdef WIN32
+#ifdef _WIN32
             if ( WSAEWOULDBLOCK != WSAGetLastError() )
 #else
             if ( errno != EWOULDBLOCK )
@@ -383,7 +351,7 @@ void OutgoingStreamWrite(SLPList* socklist, SLPDSocket* sock)
         }
         else
         {
-#ifdef WIN32
+#ifdef _WIN32
             if ( WSAEWOULDBLOCK != WSAGetLastError() )
 #else
             if ( errno != EWOULDBLOCK )
