@@ -379,10 +379,8 @@ int ProcessSrvRqst(SLPMessage message,
         errorcode = ProcessDASrvRqst(message, sendbuf, errorcode);
         if (errorcode == 0)
         {
-           /*
-           * Since we have an errorcode of 0, we were successful,
-            * and have already formed a response packet; return now.
-           */
+            // Since we have an errorcode of 0, we were successful,
+            // and have already formed a response packet; return now.
             return errorcode;
         }
 
@@ -397,10 +395,8 @@ int ProcessSrvRqst(SLPMessage message,
         errorcode = ProcessSASrvRqst(message, sendbuf, errorcode);
         if (errorcode == 0)
         {
-           /*
-           * Since we have an errorcode of 0, we were successful,
-            * and have already formed a response packet; return now.
-           */
+            // Since we have an errorcode of 0, we were successful,
+            // and have already formed a response packet; return now.
             return errorcode;
         }
 
@@ -962,50 +958,48 @@ int ProcessAttrRqst(SLPMessage message,
                                             /*  2 bytes for error code */
                                             /*  2 bytes for attr-list len */
                                             /*  1 byte for the authcount */
-    if(errorcode == 0)
-    {
-        size += db->attrlistlen;
-    
+    size += db->attrlistlen;
+
 #ifdef ENABLE_SLPv2_SECURITY
-    
-        /*------------------------------------------------------------------*/
-        /* Generate authblock if necessary or just use the one was included */
-        /* by registering agent.  Reserve sufficent space for either case.  */
-        /*------------------------------------------------------------------*/
-        if (G_SlpdProperty.securityEnabled &&
-            message->body.attrrqst.spistrlen )
+
+    /*------------------------------------------------------------------*/
+    /* Generate authblock if necessary or just use the one was included */
+    /* by registering agent.  Reserve sufficent space for either case.  */
+    /*------------------------------------------------------------------*/
+    if (G_SlpdProperty.securityEnabled &&
+        message->body.attrrqst.spistrlen )
+    {
+        if (message->body.attrrqst.taglistlen == 0)
         {
-            if (message->body.attrrqst.taglistlen == 0)
+            for (i=0; i<db->authcount;i++)
             {
-                for (i=0; i<db->authcount;i++)
+                if (SLPCompareString(db->autharray[i].spistrlen,
+                                     db->autharray[i].spistr,
+                                     message->body.attrrqst.spistrlen,
+                                     message->body.attrrqst.spistr) == 0)
                 {
-                    if (SLPCompareString(db->autharray[i].spistrlen,
-                                         db->autharray[i].spistr,
-                                         message->body.attrrqst.spistrlen,
-                                         message->body.attrrqst.spistr) == 0)
-                    {
-                        opaqueauth = db->autharray[i].opaque;
-                        opaqueauthlen = db->autharray[i].opaquelen;
-                        break;
-                    }
+                    opaqueauth = db->autharray[i].opaque;
+                    opaqueauthlen = db->autharray[i].opaquelen;
+                    break;
                 }
             }
-            else
-            {
-                errorcode = SLPAuthSignString(G_SlpdSpiHandle,
-                                              message->body.attrrqst.spistrlen,
-                                              message->body.attrrqst.spistr,
-                                              db->attrlistlen,
-                                              db->attrlist,
-                                              &generatedauthlen,
-                                              &generatedauth);
-                opaqueauthlen = generatedauthlen;
-                opaqueauth = generatedauth;
-            }
-            size += opaqueauthlen;
         }
-#endif
+        else
+        {
+            errorcode = SLPAuthSignString(G_SlpdSpiHandle,
+                                          message->body.attrrqst.spistrlen,
+                                          message->body.attrrqst.spistr,
+                                          db->attrlistlen,
+                                          db->attrlist,
+                                          &generatedauthlen,
+                                          &generatedauth);
+            opaqueauthlen = generatedauthlen;
+            opaqueauth = generatedauth;
+        }
+        size += opaqueauthlen;
     }
+#endif
+
     /*-------------------*/
     /* Alloc the  buffer */
     /*-------------------*/
@@ -1232,10 +1226,8 @@ int ProcessSrvTypeRqst(SLPMessage message,
                                             /*  2 bytes for error code */
                                             /*  2 bytes for srvtype
                                                 list length  */
-    if(errorcode == 0)
-    {
-        size += db->srvtypelistlen;
-    }
+    size += db->srvtypelistlen;
+
 
     /*------------------------------*/
     /* Reallocate the result buffer */
@@ -1441,7 +1433,7 @@ int SLPDProcessMessage(struct sockaddr_in* peerinfo,
             {
                 SLPDLogParseWarning(peerinfo, recvbuf);
             }
-
+                            
             if (header.functionid == SLP_FUNCT_SRVREG ||
                 header.functionid == SLP_FUNCT_DAADVERT )
             {
@@ -1459,7 +1451,7 @@ int SLPDProcessMessage(struct sockaddr_in* peerinfo,
                  * duplicated recvbuf,
                  */
                 SLPBufferFree(recvbuf);                    
-            }    
+            }
 
             SLPMessageFree(message);
         }
@@ -1484,9 +1476,8 @@ int SLPDProcessMessage(struct sockaddr_in* peerinfo,
                 inet_ntoa(peerinfo->sin_addr));
     }
 #endif
-
-    
-    /* Log messaged silently ignored because of an error */
+ 
+    /* Log message silently ignored because of an error */
     if(errorcode)
     {
         if (*sendbuf == 0 ||
