@@ -70,7 +70,7 @@ int EnableBroadcast(sockfd_t sockfd)
 /* returns  - zero on success                                              */
 /*-------------------------------------------------------------------------*/
 {
-#ifdef WIN32
+#ifdef _WIN32
     const char on = 1;
 #else
     const int on = 1;                                                
@@ -88,7 +88,15 @@ int SetMulticastTTL(sockfd_t sockfd, int ttl)
 /*-------------------------------------------------------------------------*/
 {
 
-#ifdef WIN32
+#if defined(linux)
+    int         optarg = ttl;
+#else
+    /* Solaris and Tru64 expect a unsigned char parameter */
+    unsigned char   optarg = (unsigned char)ttl;
+#endif
+
+
+#ifdef _WIN32
     BOOL Reuse = TRUE;
     int TTLArg;
     struct sockaddr_in  mysockaddr;
@@ -115,28 +123,10 @@ int SetMulticastTTL(sockfd_t sockfd, int ttl)
         return -1;
     }
 #else
-
-#if defined(linux)
-    int         optarg = ttl;
-    struct in_addr in;
-#else
-    /* Solaris and Tru64 expect a unsigned char parameter */
-    unsigned char   optarg = (unsigned char)ttl;
-    struct in_addr in;
-#endif
-
     if(setsockopt(sockfd,IPPROTO_IP,IP_MULTICAST_TTL,&optarg,sizeof(optarg)))
     {
         return -1;
     }
-    if(G_SlpdProperty.multicastIF)
-    {
-        if(!inet_aton(G_SlpdProperty.multicastIF, &in) ||
-           setsockopt(sockfd,IPPROTO_IP,IP_MULTICAST_IF,&in,sizeof(in)))
-        {
-            return -1;
-        }
-    }
 #endif
 
     return 0;
@@ -211,7 +201,7 @@ int BindSocketToInetAddr(int sock, struct in_addr* addr)
 /*-------------------------------------------------------------------------*/
 {
     int                 result;
-#ifdef WIN32
+#ifdef _WIN32
     char                lowat;
     BOOL                reuse = TRUE;
 #else
@@ -336,6 +326,8 @@ SLPDSocket* SLPDSocketCreateDatagram(struct in_addr* peeraddr,
     sock = SLPDSocketAlloc();
     if(sock)
     {
+        /* SLP_MAX_DATAGRAM_SIZE is as big as a datagram SLP     */
+        /* can be.                                               */
         sock->recvbuf = SLPBufferAlloc(SLP_MAX_DATAGRAM_SIZE);
         sock->sendbuf = SLPBufferAlloc(SLP_MAX_DATAGRAM_SIZE);
         if(sock->recvbuf && sock->sendbuf)
@@ -493,7 +485,7 @@ SLPDSocket* SLPDSocketCreateListen(struct in_addr* peeraddr)
                 {
                     /* Set socket to non-blocking so subsequent calls to */
                     /* accept will *never* block                         */
-#ifdef WIN32
+#ifdef _WIN32
                     fdflags = 1;
                     ioctlsocket(sock->fd, FIONBIO, &fdflags);
 #else
@@ -528,7 +520,7 @@ SLPDSocket* SLPDSocketCreateConnected(struct in_addr* addr)
 /*          be set to connect.  Return NULL on error                        */
 /*==========================================================================*/
 {
-#ifdef WIN32
+#ifdef _WIN32
     char                lowat;
     u_long              fdflags;
 #else
@@ -551,7 +543,7 @@ SLPDSocket* SLPDSocketCreateConnected(struct in_addr* addr)
     }
 
     /* set the socket to non-blocking */
-#ifdef WIN32
+#ifdef _WIN32
     fdflags = 1;
     ioctlsocket(sock->fd, FIONBIO, &fdflags);
 #else
@@ -580,7 +572,7 @@ SLPDSocket* SLPDSocketCreateConnected(struct in_addr* addr)
     }
     else
     {
-#ifdef WIN32
+#ifdef _WIN32
         if(WSAEWOULDBLOCK == WSAGetLastError())
 #else
         if(errno == EINPROGRESS)
