@@ -82,10 +82,10 @@
 #define    STREAM_WRITE_FIRST      11   + SOCKET_PENDING_IO
 #define    STREAM_WRITE_WAIT       12   + SOCKET_PENDING_IO
 
-#ifdef WIN32
-    #define CloseSocket(Arg) closesocket(Arg)
+#ifdef _WIN32
+#define CloseSocket(Arg) closesocket(Arg)
 #else
-    #define CloseSocket(Arg) close(Arg)
+#define CloseSocket(Arg) close(Arg)
 #endif
 
 /*=========================================================================*/
